/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Dashboard.
 *
 * The Initial Developer of the Original Code is
 *   Dave Raggett.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var assess = {
  start_up: function (dashboard)
  {
    // dashboard overlay passes current location to windowOpen()
    this.dashboard = dashboard;
    let status = window.arguments ? window.arguments[0] : null;

    if (status)
    {
      this.update_current_site(status);
      this.refresh_prefs(status.page_host);
    }

    let sharing = document.getElementById("sharing");
    sharing.addEventListener("command", function ()
    {
      assess.sharing_change();
    }, false);

    let simplify = document.getElementById("simplify_choices");
    simplify.addEventListener("command", function ()
    {
      assess.simplify_choices(true);
    }, false);

    let complicate = document.getElementById("let_me_choose");
    complicate.addEventListener("command", function ()
    {
      assess.expand_choices(true);
    }, false);

    let current = document.getElementById("dashboard_current");

    current.addEventListener("click", function (e)
    { 
      if (e.target.nodeName != "html:a")
         current.className = "normal";
    }, false);

    window.addEventListener("keydown", function (e)
    {
      if (e.which == 27) // Escape
        current.className = "normal";
    }, false);

    this.attach_prefs_handler("simple_choice");  // radio group
    this.attach_prefs_handler("prefs_never_block");
    this.attach_prefs_handler("prefs_do_not_track");
    this.attach_prefs_handler("prefs_ext_3rd_parties");
    this.attach_prefs_handler("prefs_ext_3rd_cookies");
    this.attach_prefs_handler("prefs_all_lasting_cookies");
    this.attach_prefs_handler("prefs_clear_flash_cookies");
    this.attach_prefs_handler("prefs_block_scripting");
    this.attach_prefs_handler("prefs_block_geolocation");
    this.attach_prefs_handler("prefs_disable_html5_pings");
    this.attach_prefs_handler("prefs_disable_referrer");
    this.attach_prefs_handler("prefs_disable_dom_storage");
    this.attach_prefs_handler("prefs_disable_flash");
    this.attach_prefs_handler("prefs_disable_java");
  },

  shut_down: function ()
  {
  },

  sharing_change: function ()
  {
    let  sharing = document.getElementById("sharing");

    if (window.dashboard_overlay)
      window.dashboard_overlay.share.set_mode(sharing.value);
  },

  refresh: function ()
  {
    this.update_current_site(this.status);
  },

  attach_prefs_handler: function (id)
  {
    let el = document.getElementById(id);
    if (el)
      el.addEventListener("command",  this.record_prefs, true);      
  },

  simplify_choices: function (active)
  {
    this.simple_choices = true;
    let detailed = document.getElementById("detailed_prefs");
    let simple = document.getElementById("simple_prefs");
    simple.style.display = "-moz-box";
    simple.style.visibilty = "visible";
    detailed.style.display = "none";
    detailed.style.visibility = "hidden";

    if (!active)
      return;

    let uri = this.uri;

    if (uri && /^http.+/.test(uri.spec) )
    {
      let prefs = 0;
      let simple_group = document.getElementById("simple_choice");

      if ( this.get_choice("prefs_ext_3rd_cookies", 4) ||
           this.get_choice("prefs_disable_html5_pings", 9) )
        prefs = 1;

      if ( this.get_choice("prefs_all_lasting_cookies", 5) ||
           this.get_choice("prefs_clear_flash_cookies", 6) ||
           this.get_choice("prefs_block_scripting", 7) ||
           this.get_choice("prefs_block_geolocation", 8) ||
           this.get_choice("prefs_disable_referrer", 10) ||
           this.get_choice("prefs_disable_dom_storage", 11) ||
           this.get_choice("prefs_disable_flash", 12) ||
           this.get_choice("prefs_disable_java", 13) )
        prefs = 2;
      
      simple_group.selectedIndex = prefs;

      let host = uri.host;
      window.database.set_prefs(host, prefs);
      setTimeout(function () { assess.refresh_prefs(host); }, 100);
    }
  },

  expand_choices:  function (active)
  {
    this.simple_choices = false;
    let detailed = document.getElementById("detailed_prefs");
    let simple = document.getElementById("simple_prefs");
    simple.style.display = "none";
    simple.style.visibilty = "hidden";
    detailed.style.display = "-moz-box";
    detailed.style.visibility = "visible";

    if (!active)
      return;

    let uri = this.uri;

    if (uri && /^http.+/.test(uri.spec))
    {
      let simple_group = document.getElementById("simple_choice");
      let prefs = simple_group.selectedIndex;

      if (prefs == 0) // carefree: don't block anything
        prefs = 3;
      else if (prefs == 1)
        prefs = 3 | (1 << 4)|(1<<9);  // thoughtful
      else  // paranoid, so block pretty much everything!
        prefs = 3 | (1<<4)|(1<<5)|(1<<6)|(1<<7)|(1<<8)|(1<<9)|(1<<10)|(1<<11)|(1<<12)|(1<<13);

      let host = uri.host;
      window.database.set_prefs(host, prefs);
      setTimeout(function () { assess.refresh_prefs(host); }, 100);
    }
  },

  record_prefs: function (evt)
  {
    // special effort to clear cookies
    // when setting preferences
    let id = this.getAttribute("id");
    let checked = this.getAttribute("checked");
    let host = assess.host;

    if (id == "prefs_all_lasting_cookies")
    {
      if (checked == "true")
         assess.query_3rd_parties(host,
           function (host, parties, offsite)
           {
             assess.purge_cookies(host, 1<<5, parties, offsite);
           });
    }
    else if (id == "prefs_ext_3rd_cookies" || id == "thoughtful")
    {
      if (checked == "true")
         assess.query_3rd_parties(host,
           function (host, parties, offsite)
           {
             assess.purge_cookies(host, 1<<4, parties, offsite);
           });
    }
    else if (id == "paranoid")
    {
      if (checked == "true")
         assess.query_3rd_parties(host,
           function (host, parties, offsite)
           {
             assess.purge_cookies(host, (1<<4 | 1<<5), parties, offsite);
           });
    }

    assess.save_prefs(host);
  },

  // query for list of 3rd parties for this site, assumes that
  // database.log_3rd_parties(status); has already taken effect
  query_3rd_parties: function (host, action)
  {
    let database = window.database;
    let query = "SELECT DISTINCT third_party, offsite FROM  parties " +
                "WHERE page_host = " + database.quote(host);

    let handler = 
    {
      parties: [],
      offsite: [],

      process_row: function (row)
      {
        this.parties.push(row.getResultByName("third_party"));
        this.offsite.push(row.getResultByName("offsite"));        
      },

      finalize: function () 
      {
        action(host, this.parties, this.offsite);
      }
    };

    database.select(query, database.db, handler);
  },

  // code copied from misc.rate_cookies()
  purge_cookies: function (host, prefs, parties, offsite)
  {
    let cm = Components.classes['@mozilla.org/cookiemanager;1'].
      getService(Components.interfaces.nsICookieManager2);

    let cookies = cm.enumerator;
    let lasting_cookies = [];
    let session_cookies = [];
    let int_3rd_lasting_cookies = [];
    let int_3rd_session_cookies = [];
    let ext_3rd_lasting_cookies = [];
    let ext_3rd_session_cookies = [];

    // iterate through cookiemanager's list of current cookies
    // and check against first and all third party sites, this
    // would require *many* database queries using SQLite
    while (cookies.hasMoreElements())
    {
      let cookie = cookies.getNext();
      cookie.QueryInterface(Components.interfaces.nsICookie2);

      // cookie matching web page host
      if (host.indexOf(cookie.host) >= 0)
      {
        if (cookie.isSession)
          session_cookies.push(cookie);
        else
          lasting_cookies.push(cookie);
      }
      else  // check for 3rd party cookies
      {
        // pump up the cost!
        for (let i in parties)
        {
          if (parties[i].indexOf(cookie.host) >= 0)
          {
            if (offsite[i] == 2)
            {
              if (cookie.isSession)
                ext_3rd_session_cookies.push(cookie);
              else
                ext_3rd_lasting_cookies.push(cookie);
            }
            else
            {
              if (cookie.isSession)
                int_3rd_session_cookies.push(cookie);
              else
                int_3rd_lasting_cookies.push(cookie);
            }
          }
        }
      }
    }

    let int_3rd_parties = 0;
    let ext_3rd_parties = 0;

    for (let i = 0; i < offsite.length; ++i)
    {
      if (offsite[i] == 2)
        ++ext_3rd_parties;
      else
        ++int_3rd_parties;
    }

    if (prefs & (1<<5))  // purge all lasting cookies
    {
      for (let i = 0; i < lasting_cookies.length; ++i)
      {
        cookie = lasting_cookies[i];
        cm.remove(cookie.host, cookie.name, cookie.path, false);
      }

      for (let i = 0; i < int_3rd_lasting_cookies.length; ++i)
      {
        cookie = int_3rd_lasting_cookies[i];
        cm.remove(cookie.host, cookie.name, cookie.path, false);
      }

      for (let i = 0; i < ext_3rd_lasting_cookies.length; ++i)
      {
        cookie = ext_3rd_lasting_cookies[i];
        cm.remove(cookie.host, cookie.name, cookie.path, false);
      }
    }
    else
    {
      if (prefs & (1<<4))  // purge all 3rd party cookies
      {
        for (let i = 0; i < ext_3rd_session_cookies.length; ++i)
        {
          cookie = ext_3rd_session_cookies[i];
          cm.remove(cookie.host, cookie.name, cookie.path, false);
        }

        for (let i = 0; i < ext_3rd_lasting_cookies.length; ++i)
        {
          cookie = ext_3rd_lasting_cookies[i];
          cm.remove(cookie.host, cookie.name, cookie.path, false);
        }
      }
    }
  },

  refresh_prefs: function (host)
  {
    let database = window.database;
    let db = database.db;
    let simple_group = document.getElementById("simple_choice");

    // bit 0-1 encode simple preferences as follows
    // 0 = carefree 1 = thoughtful 2 = paranoid 3 = advanced
    let prefs = database.get_prefs(host, db);

    //alert("prefs = " + prefs + " " + ((prefs & 3)<3 ? "complex" : "simple") );

    // if no preferences are recorded for this site
    if (typeof(prefs) == "undefined")
    {
      this.simplify_choices();
      simple_group.selectedIndex = 0;
    }
    else // update UI to reflect recorded preferences
    {
      let simple = (prefs & 3);

      if (simple < 3)
      {
        this.simplify_choices();
        simple_group.selectedIndex = simple;
      }
      else // advanced choices
      {
        this.expand_choices(false);
        this.mark_choice("prefs_never_block", prefs, 2);
        this.mark_choice("prefs_ext_3rd_parties", prefs, 3);
        this.mark_choice("prefs_ext_3rd_cookies", prefs, 4);
        this.mark_choice("prefs_all_lasting_cookies", prefs, 5);
        this.mark_choice("prefs_clear_flash_cookies", prefs, 6);
        this.mark_choice("prefs_block_scripting", prefs, 7);
        this.mark_choice("prefs_block_geolocation", prefs, 8);
        this.mark_choice("prefs_disable_html5_pings", prefs, 9);
        this.mark_choice("prefs_disable_referrer", prefs, 10);
        this.mark_choice("prefs_disable_dom_storage", prefs, 11);
        this.mark_choice("prefs_disable_flash", prefs, 12);
        this.mark_choice("prefs_disable_java", prefs, 13);
        this.mark_choice("prefs_use_by_default", prefs, 14);
        this.mark_choice("prefs_do_not_track", prefs, 15);
      }
    }
  },

  save_prefs: function (host)
  {
    let database = window.database;
    let db = database.db;
    let simple_group = document.getElementById("simple_choice");
    let prefs = 0;

    if (this.simple_choices)
      prefs = simple_group.selectedIndex;
    else
    {
      prefs = 3;
      prefs |= this.get_choice("prefs_never_block", 2);
      prefs |= this.get_choice("prefs_ext_3rd_parties", 3);
      prefs |= this.get_choice("prefs_ext_3rd_cookies", 4);
      prefs |= this.get_choice("prefs_all_lasting_cookies", 5);
      prefs |= this.get_choice("prefs_clear_flash_cookies", 6);
      prefs |= this.get_choice("prefs_block_scripting", 7);
      prefs |= this.get_choice("prefs_block_geolocation", 8);
      prefs |= this.get_choice("prefs_disable_html5_pings", 9);
      prefs |= this.get_choice("prefs_disable_referrer", 10);
      prefs |= this.get_choice("prefs_disable_dom_storage", 11);
      prefs |= this.get_choice("prefs_disable_flash", 12);
      prefs |= this.get_choice("prefs_disable_java", 13);
      prefs |= this.get_choice("prefs_use_by_default", 14);
      prefs |= this.get_choice("prefs_do_not_track", 15);
    }

    database.set_prefs(host, prefs);

    if (prefs & (1 << 14))
      database.set_prefs("default", prefs);

    let global_prefs = 0;
    // now get global choices and save to database under "global"
  },

  get_choice: function (id, code)
  {
    let checkbox = document.getElementById(id);

    if (checkbox.getAttribute("checked") == "true")
      return (1 << code);

    return 0;
  },

  mark_choice: function (id, prefs, code)
  {
    let checkbox = document.getElementById(id);
    let state = prefs & (1 << code);
    checkbox.setAttribute("checked", state ? "true" : "false");
  },

  update_current_site: function (status)
  {
    let uri = status.uri;
    let host = status.page_host;
    let path = uri.path;

    if (status.new_url)
    {
      host = dashboard.extract_host(status.new_url);
    }

    this.status = status;
    this.uri = uri;
    this.host = host;
    this.path = path;

    let textbox = document.getElementById("query_textbox");
    textbox.value=host;
    //dashboard.history.set();

    let sd = document.getElementById("site_description");
    let p = document.getElementById("curhost");

    if (p)
      p.innerHTML = host ? host : "Blank page, no host";

    let singleton_window =
        Components.classes["@mozilla.org/appshell/appShellService;1"].
          getService(Components.interfaces.nsIAppShellService).hiddenDOMWindow;

    p.style.backgroundImage = singleton_window.dashboard_icon;

    if (host)
    {
      this.refresh_prefs(host);
      this.show_status(status);
      sd.className = "";
    }
    else
      sd.className = "hidden";
  },

  show_policy: function ()
  {
    let host = assess.host;
    let path = assess.path;
    let policy_uri = "http://"+host+"/w3c/p3p.xml";

    dashboard_p3p.render_policy(policy_uri, path, 0);
  },

  show_status: function (status)
  {
    let list = document.getElementById("curstatus");

    // clear status
    let child = list.firstChild;

    while (child)
    {
      list.removeChild(child);
      child = list.firstChild;
    }

    let localize = dashboard.localize;

    this.status_count = 0;

    this.add_item(list, status.session_cookies,
              "access.caption.cookies.session",
              "access.tooltip.cookies.session");
    this.add_item(list, status.lasting_cookies,
              "access.caption.cookies.lasting",
              "access.tooltip.cookies.lasting");
    this.add_item(list, status.flash_cookies,
              "access.caption.cookies.flash",
              "access.tooltip.cookies.flash");

    this.add_item(list, status.int_3rd_parties,
              "access.caption.internal.third.parties",
              "access.tooltip.internal.third.parties");

    this.add_item(list, status.ext_3rd_parties,
              "access.caption.external.third.parties",
              "access.tooltip.external.third.parties");

    this.add_item(list, status.int_3rd_party_session_cookies,
              "access.caption.internal.third.parties.cookies.session",
              "access.tooltip.internal.third.parties.cookies.session");
    this.add_item(list, status.int_3rd_party_lasting_cookies,
              "access.caption.internal.third.parties.cookies.lasting",
              "access.tooltip.internal.third.parties.cookies.lasting");
    this.add_item(list, status.int_3rd_party_flash_cookies,
              "access.caption.internal.third.parties.cookies.flash",
              "access.tooltip.internal.third.parties.cookies.flash");

    this.add_item(list, status.ext_3rd_party_session_cookies,
              "access.caption.external.third.parties.cookies.session",
              "access.tooltip.external.third.parties.cookies.session");
    this.add_item(list, status.ext_3rd_party_lasting_cookies,
              "access.caption.external.third.parties.cookies.lasting",
              "access.tooltip.external.third.parties.cookies.lasting");
    this.add_item(list, status.ext_3rd_party_flash_cookies,
              "access.caption.external.third.parties.cookies.flash",
              "access.tooltip.external.third.parties.cookies.flash");

    this.add_item(list, status.dom_storage,
              "access.caption.dom_storage",
              "access.tooltip.dom_storage");

    child = this.add_item(list, status.geo_permission,
              "access.caption.geolocation",
              "access.tooltip.geolocation");

    if (child)
      child.innerHTML = localize("access.caption.geolocation");

    if (status.html5_pings)
    {
      let item = document.createElementNS("http://www.w3.org/1999/xhtml", "li");
      list.appendChild(item);

      if (status.html5_pings == 1)
        item.innerHTML = dashboard.localize("access.caption.ping1");
      else
        item.innerHTML = status.html5_pings + " "  + dashboard.localize("access.caption.ping");

      item.setAttribute("tooltiptext", dashboard.localize("access.tooltip.ping"));
      this.status_count++;
    }

    if (status.invisible_images)
    {
      let item = document.createElementNS("http://www.w3.org/1999/xhtml", "li");
      list.appendChild(item);

      if (status.invisible_images == 1)
        item.innerHTML = dashboard.localize("access.caption.images1");
      else
        item.innerHTML = status.invisible_images + " "  + dashboard.localize("access.caption.images");

      item.setAttribute("tooltiptext", dashboard.localize("access.tooltip.images"));
      this.status_count++;
    }

    if (status.suspicious_urls)
    {
      let item = document.createElementNS("http://www.w3.org/1999/xhtml", "li");
      list.appendChild(item);

      if (status.suspicious_urls == 1)
        item.innerHTML = dashboard.localize("access.caption.bad_urls1");
      else
        item.innerHTML = status.suspicious_urls + " "  + dashboard.localize("access.caption.bad_urls");

      item.setAttribute("tooltiptext", dashboard.localize("access.tooltip.bad_urls"));
      this.status_count++;
    }

    if (status.p3p)
    {
      let item = document.createElementNS("http://www.w3.org/1999/xhtml", "li");
      list.appendChild(item);
      item.innerHTML = dashboard.localize("access.a") +
           " <html:a tooltiptext='"+
           dashboard.localize("access.tooltip.p3p") +
           "' onclick='assess.show_policy()'>" +
           dashboard.localize("access.caption.p3p") + "</html:a>";
      this.status_count++;
    }

    if (status.old_host && status.old_host != status.page_host)
    {
      let item = document.createElementNS("http://www.w3.org/1999/xhtml", "li");
      list.appendChild(item);

      item.innerHTML = dashboard.localize("access.caption.redirect") +
        "<html:br/>" + status.old_host;

      item.setAttribute("tooltiptext", dashboard.localize("access.tooltip.redirect"));
    }

    let has = document.getElementById("website_has");

    if (this.status_count == 0)
      has.innerHTML = "";
    else
      has.innerHTML = dashboard.localize("access.site")+":";

    let sd = document.getElementById("site_description");

    if (this.status_count)
      sd.className = "";
    else
      sd.className = "hidden";
  },

  show_help: function (info)
  {
     let help_positioner = document.getElementById("help_positioner");
     let help_box = document.getElementById("context_help_box");
     let help_label = document.getElementById("context_help_label");
     let help_text = document.getElementById('dashboard-strings').getString(info);
     help_label.innerHTML = "";
     help_label.appendChild(document.createTextNode(help_text));
     help_box.openPopup(help_positioner, "after_start"); 
  },

  hide_help: function ()
  {
     let help_box = document.getElementById("context_help_box");
     help_box.hidePopup();
  },

  add_item: function (list, count, what, help)
  {
    if (count > 0)
    {
      let item = document.createElementNS("http://www.w3.org/1999/xhtml", "li");
      list.appendChild(item);
      let num;

      if (count == 1)
        item.innerHTML = dashboard.localize(what+"1");
      else
        item.innerHTML = count + " " + dashboard.localize(what);

      item.setAttribute("tooltiptext", dashboard.localize(help));
/*
      var help_button = document.createElementNS("http://www.w3.org/1999/xhtml", "img");
      help_button.setAttribute("src", 'chrome://dashboard-common/skin/help.png');
      help_button.addEventListener("mouseover", function () { assess.show_help(help); }, false);
      help_button.addEventListener("mouseout", function () { assess.hide_help(); }, false);
      item.appendChild(help_button);
*/
      this.status_count++;
      return item;
    }

    return null;
  }
}
