/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Dashboard.
 *
 * The Initial Developer of the Original Code is
 *   Dave Raggett.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var console = {
  service: null,
  log: function(msg)
  {
    if (!console.service)
      console.service = Components.classes["@mozilla.org/consoleservice;1"]
                     .getService(Components.interfaces.nsIConsoleService);
    console.service.logStringMessage(msg);
  }
};

var dashboard = {
  on_load: function ()
  {
    window.dashboard = this;
    var parent_window = window.arguments ? window.arguments[1] : null;

    this.strings = document.getElementById("dashboard-strings");

    // URL for server to share data with
    let share_to = document.getElementById("share_to");
    share_to.addEventListener("change", function (e)
    {
       // Get the "extensions.myext." branch
       let prefs = Components.classes["@mozilla.org/preferences-service;1"]
                    .getService(Components.interfaces.nsIPrefService);
       prefs = prefs.getBranch("extensions.privacydashboard.");

       let value = e.target.value;
       if (!/^http.+/.test(value))
       {
         value = "http://" + value;
         e.target.value = value;
       }

       prefs.setCharPref("share.with", value);
       return false;
    }, false);

    // hypertext link in XUL dialog normally opens in that dialog
    // we override that behavior here to force load into new tab
    // on the parent browser window, and to then select that tab
    let link = document.getElementById("dashboard_website");
    link.addEventListener("click", function (e)
    {
       let browser = parent_window.gBrowser;
       browser.selectedTab =  browser.addTab("http://code.w3.org/privacy-dashboard/");
       e.cancel = true;
       e.preventDefault();
       return false;
    }, false);

    link = document.getElementById("primelife_website");
    link.addEventListener("click", function (e)
    {
       let browser = parent_window.gBrowser;
       browser.selectedTab =  browser.addTab("http://www.primelife.eu");
       e.cancel = true;
       e.preventDefault();
       return false;
    }, false);

    // set event handler for location mode buttons
    let loc_mode_el = document.getElementById("location_mode");
    loc_mode_el.addEventListener("select", function ()
    {
      dashboard.update_location();
    }, false);

    // hook up handlers for back/forward buttons
    let leftbutton = document.getElementById("backward");
    leftbutton.addEventListener("command", function ()
    {
      dashboard.history.back();
    }, false);

    let rightbutton = document.getElementById("forward");
    rightbutton.addEventListener("command", function ()
    {
      dashboard.history.forward();
    }, false);

    // hook up button for executing queries
    let execbutton = document.getElementById("exec_query_button");
    execbutton.addEventListener("command", function ()
    {
      dashboard.exec_query(true);
    }, false);

    let query_tree = document.getElementById("query_tree");

    // set click handler for results table
    query_tree.addEventListener("click", this.results_click, false);

    // make Enter key execute queries for data track 
    let query_menu = document.getElementById('query_menu');

    query_menu.addEventListener("keydown", function (e)
    {
      if (e.which == 13)
        dashboard.exec_query(true);
    }, false);

    let tabs = document.getElementById("dashboard-tabs");

    // when returning to current website tab
    // set the datatrack text box to current host
    tabs.addEventListener("select", function (e)
    {
       if (e.target.selectedIndex == 2)
       {
         let textbox = document.getElementById("query_textbox");
         textbox.value = assess.host;
         //dashboard.history.set();
       }
    }, false);

    let textbox = document.getElementById("query_textbox");

    textbox.addEventListener("keydown", function (e)
    {
      if (e.which == 13)
        dashboard.exec_query(true);
    }, false);

    // adapt dialog window height for smaller displays
    // this particular technique may be too late to work?
    if (window.screen.height < 1000)
       query_tree.setAttribute("rows", "10");

    // activate the current website tab
    let tabbox = document.getElementsByTagName("tabbox");
    tabbox[0].selectedIndex = 2;

    // activate helper object for assessing current site
    assess.start_up(this);

    // copy version number into about tab

    try
    {
      // Firefox 4 and later; Mozilla 2 and later
      Components.utils.import("resource://gre/modules/AddonManager.jsm");
      AddonManager.getAddonByID("dashboard@dave.raggett",
        function(addon)
        {
          let about_heading = document.getElementById("dashboard_heading");
          about_heading.innerHTML = about_heading.innerHTML + " " + addon.version;
        });
    }
    catch (e)
    {
      // Firefox 3.6 and before; Mozilla 1.9.2 and before
      let gExtensionManager = Components.classes["@mozilla.org/extensions/manager;1"]
                        .getService(Components.interfaces.nsIExtensionManager);
      let addon = gExtensionManager.getItemForID("dashboard@dave.raggett");
      let about_heading = document.getElementById("dashboard_heading");
      about_heading.innerHTML = about_heading.innerHTML + " " + addon.version;
    }

    // record whether site supports P3P

    // try to find directory for flash storage
    //this.set_flash_dir();
    //var misc = window.database.misc;
    //misc.scan_flash_storage();  // test of reading folders

    // demo Firefox's ability to find out where you are (see about tab)
    this.update_location();
  },

  on_unload: function ()
  {
    // de-activate helper object for assessing current site
    assess.shut_down();

    // ensure parent browser knows
    let context = window.dashboard_overlay.context;
    context.dashboard_dialog = null;
  },

  localize: function (key)
  {
    try
    {
      return dashboard.strings.getString(key);
    }
    catch (e)
    {
      alert("Couldn't get localized string for " + key);
      return key;
    }
  },

  history: 
  {
    here: -1,
    history: [],

    button_state: function (state)
    {
      let backward = document.getElementById('backward');
      let forward = document.getElementById('forward');
      backward.className = this.here >= 0 ? "" : "inactive";
      forward.className = this.here < this.history.length - 1 ? "" : "inactive";

      let index = state ? parseInt(state.query) : 0;
      let query_param = document.getElementById('query_textbox');
      query_param.disabled = index < 8 || index == 12;
    },

    execute: function (state)
    {
      let query_menu = document.getElementById('query_menu');
      let query_param = document.getElementById('query_textbox');

      query_menu.value = "" + state.query
      query_param.value = state.value;
      dashboard.exec_query(false);
      this.button_state(state);
      console.log("exec: " + this.here + ", " + JSON.stringify(state));
    },


    back: function (e)
    {
      if (this.here > 0)
      {
        let state = this.history[--this.here];
        this.execute(state);
      }
      else
      {
        let tree = document.getElementById('query_tree');
        let treecols = dashboard.get_element(tree, "treecols");
        let treechildren = dashboard.get_element(tree, "treechildren");

        dashboard.strip_content(treecols);
        dashboard.strip_content(treechildren);
        let treecol = document.createElement("treecol");
        treecol.setAttribute("flex", 1);
        treecols.appendChild(treecol);
        this.here = -1;
        this.button_state();
      }
    },

    forward: function (e)
    {
      if (this.here < this.history.length - 1)
      {
        let state = this.history[++this.here];
        this.execute(state);
      }
    },

    set: function ()
    {
      let query_menu = document.getElementById('query_menu');
      let query_param = document.getElementById('query_textbox');

      let state = { "value": query_param.value, "query": query_menu.value };
      this.history[++this.here] = state;
      this.history = this.history.slice(0, this.here + 1);
      this.button_state(state);
    },

    clear: function ()
    {
      this.here = 0;
      this.history = [];
      this.button_state();
    }
  },

  results_click: function (e)
  {
    let query_tree = document.getElementById("query_tree");
    let textbox = document.getElementById("query_textbox");
    try
    {
      let treechildren = e.target;
      let treeitem = treechildren.firstChild;
      let i, selected = query_tree.currentIndex;

      for (i = 0; i < selected; ++i)
        treeitem = treeitem.nextSibling;

      let treerow = treeitem.firstChild
      let treecell = treerow.firstChild;
      textbox.value = treecell.getAttribute("label");
      //dashboard.history.set();
    }
    catch (e) { }
  },

  // pass this on to helper object
  // note that the doc uri may differ from the uri passed here
  // which is obtained from the onLocationChange event
  update_current_site: function (status) {
    console.log("dashboard.update_current_site: " + status.page_host);
    assess.update_current_site(status);
  },

  show_status: function ()
  {
    assess.show_status();
  },

  update_location: function ()
  {
    let loc_tab = document.getElementById("dashboard_location");
    let loc_mode_el = document.getElementById("location_mode");
    let mode = loc_mode_el.selectedIndex;

    let sitelist = document.getElementById("site_list");

    while (sitelist.firstChild)
      sitelist.removeChild(sitelist.firstChild);

    if (mode == 0) // show list of sites with permission to access geolocation
    {
      loc_tab.className = "show_sites";
      let p = document.createElementNS("http://www.w3.org/1999/xhtml", "p");
      sitelist.appendChild(p); 
      p.innerHTML = dashboard_overlay.localize("dashboard.loc7");

      let vbox = document.createElementNS(
        "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "vbox");
      sitelist.appendChild(vbox);
      vbox.setAttribute("style", "overflow:auto;height:400px");

      this.show_geo_sites(vbox);
    }
    else // show geographic location on map
    {
      loc_tab.className = "show_map";
      let map = document.getElementById("map");

      if (map.getAttribute("src") == "")
      {     
        let pos = document.getElementById("geoloc");
        pos.innerHTML = "";

        let geo_prefs = Components.classes["@mozilla.org/preferences-service;1"]
           .getService(Components.interfaces.nsIPrefService).getBranch("geo.");

        let geo_pref = geo_prefs.getBoolPref("enabled");
        geo_prefs.setBoolPref("enabled", true);

        try
        {
          let geolocation = Components.classes["@mozilla.org/geolocation;1"]  
                      .getService(Components.interfaces.nsIDOMGeoGeolocation);

          if (geolocation)
            geolocation.getCurrentPosition(this.set_gelocation,
                       this.geo_error, {enableHighAccuracy:true, timeout:8000});
          else
            alert("couldn't get geolocation interface");
        }
        catch (e)
        {
          let error = {};
          error.code = 3;
          this.geo_error(error);
          alert(e);
        }

        geo_prefs.setBoolPref("enabled", geo_pref);
      }
    }
  },

  show_geo_sites: function (vbox)
  {
    let handler =
    {
      hosts: [],
      permissions: [],
      count: 0,
      vbox: vbox,

      process_row: function (row)
      {
        let host = row.getResultByName("host");
        this.hosts[this.count] = host;
        let permission = row.getResultByName("permission");
        this.permissions[this.count] = permission;
        this.count++;
      },

      // ready to revoke selected geolocation permissions
      finalize: function ()
      {
        if (this.count == 0)
        {
          let sitelist = this.vbox.parentNode;
          for (let p = sitelist.firstChild; p; p = p.nextSibling)
          {
            if (p.nodeType == "1" && p.nodeName == "p")
            {
              p.innerHTML = dashboard_overlay.localize("dashboard.loc8");
              break;
            }
          }
        }

        for (let i = 0; i < this.count; ++i)
        {
          var checkbox = document.createElementNS(
            "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
            "checkbox");
          this.vbox.appendChild(checkbox);
          let allow = dashboard_overlay.localize("dashboard.loc9");
          let deny = dashboard_overlay.localize("dashboard.loc10");
          checkbox.setAttribute("label", this.hosts[i] + ", " +
                   (this.permissions[i] ? allow : deny));
          checkbox.setAttribute("checked", "false");
          checkbox.host = this.hosts[i];
          checkbox.addEventListener("command", function ()
          {
            let database = dashboard_overlay.database;
            database.permissions.executeSimpleSQL(
              "DELETE FROM moz_hosts WHERE type = 'geo' AND host = " +
                   database.quote(this.host));
            checkbox.parentNode.removeChild(checkbox);
          }, false);
        }

        this.hosts = null;
        this.permissions = null;
      }
    };

    let database = dashboard_overlay.database;
    let sql = "SELECT host, permission FROM moz_hosts WHERE type = 'geo'";

    database.select(sql, database.permissions, handler);
  },

  // show where you are on the dashboard location tab
  set_gelocation: function (position)
  {
    let loc_mode_el = document.getElementById("location_mode");
    let mode = loc_mode_el.selectedIndex;

    if (mode == 1)
    {
      let pos = document.getElementById("geoloc");
      let address = "", addr = position.address;
      let latitude = dashboard.round(position.coords.latitude, 1000);
      let longitude = dashboard.round(position.coords.longitude, 1000);

      if (addr)
        address = "<html:br/>" + addr.street + ", " + addr.city + ", " + addr.country;

      pos.innerHTML = dashboard.localize("dashboard.loc1")+": " + latitude + " " +
                    dashboard.localize("dashboard.loc2")+": " + longitude + address;

      let map = document.getElementById("map");
      let gmap = "http://maps.google.com/maps/api/staticmap?";
      let gparams = "sensor=false&zoom=14&size=600x380&maptype=roadmap"
      let loc = latitude + "," + longitude;
      map.setAttribute("src", gmap+gparams+"&center=" + loc + "&markers=" + loc);
    }
  },

  geo_error: function (error)
  {
    let loc_mode_el = document.getElementById("location_mode");
    let mode = loc_mode_el ? loc_mode_el.selectedIndex : 0;

    if (mode == 1)
    {
      let pos = document.getElementById("geoloc");
      pos.innerHTML = "<html:em>" + dashboard.localize("dashboard.loc3") +
        (error.code == 1 ? " (" + dashboard.localize("dashboard.loc4")+")" : 
        (error.code == 3 ?  " ("+ dashboard.localize("dashboard.loc5")+")"
        : "")) + ".</html:em> " + dashboard.localize("dashboard.loc6");

      let map = document.getElementById("map");
      map.setAttribute("src", "");
    }
  },

  select_query: function ()
  {
    let caption;
    let query_menu = document.getElementById('query_menu');
    let label = document.getElementById("query_textbox_label");
    let textbox = document.getElementById("query_textbox");
    let autocomplete = "history";
    let unused = "2. unused";

    switch (parseInt(query_menu.value))
    {
       case 0: // Which domains use long lasting cookies?
           caption = unused;
         break;

       case 1: // Which domains use session cookies?
           caption = unused;
         break;

       case 2: // Which domains use flash cookies?
           caption = unused;
         break;

       case 3: // Which domains use DOM storage?
           caption = unused;
         break;

       case 4: // Which domains use invisible images?
           caption = unused;
         break;

       case 5: // Which domains use html5 ping attributes?
           caption = unused;
         break;

       case 6: // Which domains use P3P?
           caption = unused;
         break;

       case 7: // Which domains are 3rd parties?
           caption = unused;
         break;

       case 8: // Which websites use a given 3rd party?
           caption = dashboard.localize("dashboard.query1")+":";
         break;

       case 9: // Which internal 3rd parties are used by a given domain?
           caption = dashboard.localize("dashboard.query1")+":";
         break;

       case 10: // Which external 3rd parties are used by a given domain?
           caption = dashboard.localize("dashboard.query1")+":";
         break;

       case 11: // What cookies are used by a given domain?
           caption = dashboard.localize("dashboard.query1")+":";
         break;

       case 12: // Which sites have I given access to my location?
           caption = unused;
         break;

       case 13: // What data has been sent to a given domain?
           caption = dashboard.localize("dashboard.query1")+":";
         break;

       case 14: // Which domains a given datum value has been sent to?
           caption = dashboard.localize("dashboard.query2")+":";
         break;

       case 15: // Which domains a given datum name has been sent to?
           caption = dashboard.localize("dashboard.query3")+":";
           autocomplete = "form-history";  // WRONG - FIX ME
         break;

       case 16: // Which datum names are used for a given value?
           caption = dashboard.localize("dashboard.query4")+":";
           autocomplete = "form-history";
         break;
    }

    label.setAttribute("value", caption);
    textbox.disabled = (caption == unused ? true : false);
    textbox.setAttribute("autocompletesearch", autocomplete);
    //dashboard.history.set();
  },

  exec_query: function (history)
  {
    let query_menu = document.getElementById('query_menu');
    let query_param = document.getElementById('query_textbox');
    let query, names, param = query_param.value;
    let database = window.database;
    let db = database.db;

    query = names = null;
    let dom_storage_query = false;

    // test needed to avoid calling set from back or forward actions
    if (history)
      dashboard.history.set();

    switch (parseInt(query_menu.value))
    {
       case 0: // Which hosts use long lasting cookies?
           //this.show_cookies(false);  // use cookie manager to enumerate cookies
           query = "SELECT DISTINCT host FROM site_info WHERE lasting_cookies <> 0";
           names = ["host"];
         break;

       case 1: // Which hosts use session cookies?
           //this.show_cookies(true);
           query = "SELECT DISTINCT host FROM site_info WHERE session_cookies <> 0";
           names = ["host"];
         break;

       case 2: // Which hosts use flash cookies?
           this.show_flash_cookies();
         break;

       case 3: // Which hosts use DOM storage?
           db = database.dom_storage;
           dom_storage_query = true;
           query = "SELECT scope, key, value, secure FROM webappsstore2";
           names = ["scope", "key", "value", "secure"];
         break;

       case 4: // Which hosts use invisible images?
           query = "SELECT host FROM site_info WHERE invisible_images <> 0";
           names = ["host"];
         break;

       case 5: // Which hosts use html5 ping attributes?
           query = "SELECT host FROM site_info WHERE html5_pings <> 0";
           names = ["host"];
         break;

       case 6: // Which hosts use P3P?
           query = "SELECT host FROM site_info WHERE p3p <> 0";
           names = ["host"];
         break;

       case 7: // Which domains are 3rd parties?
           query = "SELECT DISTINCT third_party FROM parties WHERE offsite <> 0";
           names = ["third_party"];
         break;

       case 8: // Which websites use a given 3rd party?
           param = this.extract_host(param);
           query = "SELECT DISTINCT page_host FROM parties WHERE offsite <> 0 " +
                   "AND third_party LIKE '%" + dashboard.escape1(param) + "'";
           names = ["page_host"];
         break;

       case 9: // Which internal 3rd parties are used by a given host?
           param = this.extract_host(param);
           query = "SELECT DISTINCT third_party FROM parties WHERE offsite = 1 " +
                   "AND page_host LIKE '%" + dashboard.escape1(param) + "'";
           names = ["third_party"];
         break;

       case 10: // Which external 3rd parties are used by a given host?
           param = this.extract_host(param);
           query = "SELECT DISTINCT third_party FROM parties WHERE offsite = 2 " +
                   "AND page_host LIKE '%" + dashboard.escape1(param) + "'";
           names = ["third_party"];
         break;

       case 11: // What cookies are used by a given domain?
           this.show_cookies_for_host(param);  // from cookie manager
         break;

       case 12: // Which hosts have I given access to my location?
           db = database.permissions;
           query = "SELECT host, permission FROM moz_hosts WHERE type = 'geo'";
           names = ["host", "permission"];
         break;

       case 13: // What data has been sent to a given host?
           param = this.extract_host(param);
           query = "SELECT name, value, form, time FROM http_data WHERE host LIKE '%"
                   + dashboard.escape2(param) + "'";
           names = ["name", "value", "form", "time"];
         break;

       case 14: // Which hosts a given datum value has been sent to?
           query = "SELECT host, time FROM http_data WHERE value = '"
                   + dashboard.escape1(param) + "'";
           names = ["host", "time"];
         break;

       case 15: // Which hosts a given datum name has been sent to?
           query = "SELECT host, time FROM http_data WHERE name = '"
                   + dashboard.escape1(param) + "'";
           names = ["host", "time"];
         break;

       case 16: // Which datum names are used for a given value?
           query = "SELECT DISTINCT name FROM http_data WHERE value ='"
                   + dashboard.escape1(param) + "'";
           names = ["name"];
         break;
    }

    if (query)
    {
      let tree = document.getElementById('query_tree');
      let treecols = this.get_element(tree, "treecols");
      let treechildren = this.get_element(tree, "treechildren");

      this.strip_content(treecols);
      this.strip_content(treechildren);
      this.create_columns(treecols, names);
      var self = this;

      let handler =
      {
        process_row: function (row)
        {
          let treeitem = document.createElement("treeitem");
          let treerow = document.createElement("treerow");
          treeitem.appendChild(treerow);

          for (let i = 0; i < names.length; ++i)
          {
            let cell = document.createElement("treecell");
            let name = names[i];
            let value = row.getResultByName(name);

            // ugly hack to recover host name from scope field
            if (dom_storage_query && name == "scope")
              value = this.scope_to_host(value);

            cell.setAttribute("label", value);
            treerow.appendChild(cell);
          }

          treechildren.appendChild(treeitem);
        },

        finalize: function ()
        {
        }
      };

      database.select(query, db, handler);
    }
  },

  create_columns: function (treecols, names)
  {
    let treecol, splitter;

    for (let i = 0; i < names.length; ++i)
    {
      treecol = document.createElement("treecol");
      treecol.setAttribute("label", names[i]);
      treecol.setAttribute("flex", "1");
      treecols.appendChild(treecol);

      splitter = document.createElement("splitter");
      splitter.setAttribute("class", "tree-splitter");
      splitter.setAttribute("resizeafter", "grow");
      treecols.appendChild(splitter);
    }
  },

  // generate host from mozilla's reverse syntax for scope
  scope_to_host: function (s)
  {
    let i = s.lastIndexOf(':');
    let port = s.substr(i);
    i = s.indexOf(':') - 1;
    let host = s.substr(0, i);
    host = host.split('').reverse();
    return (host.join(''))+port;
  },

  extract_host: function (url)
  {
    let start = url.indexOf("http://");

    if (start == 0)
      url = url.substr(7);

    let end = url.indexOf("/");
 
    if (end > 0)
      url = url.substr(0, end);

    let port = url.indexOf(":")

    if (port > 0)
      url = url.substr(0, port);

    return url;
  },

  set_flash_dir: function ()
  {
    // find and present flash storage directory
    let el = document.getElementById("flashdir");
    let misc = window.database.misc;
    let dir = misc.get_flash_dir();

    if (el && dir)
      el.innerHTML = "Flash storage directory: " + dir.path;

    // do same for the Firefox profile directory
    // for webappsstore.sqlite used for DOM storage
    el = document.getElementById("profiledir");
    dir = Components.classes["@mozilla.org/file/directory_service;1"].
               getService(Components.interfaces.nsIProperties).
               get("ProfD", Components.interfaces.nsIFile);

    if (el && dir)
      el.innerHTML = "Firefox profile directory: " + dir.path;
  },

  show_flash_cookies: function ()
  {
    let names = ["Host", "File", "Size", "Last Modified"];
    let tree = document.getElementById('query_tree');
    let treecols = this.get_element(tree, "treecols");
    let treechildren = this.get_element(tree, "treechildren");

    this.strip_content(treecols);
    this.strip_content(treechildren);
    this.create_columns(treecols, names);

    let misc = window.database.misc;
    let hosts = misc.scan_flash_storage();

    for (let host in hosts)
    {
      let files = hosts[host];

      for (let j = 0; j < files.length; ++j)
      {
        let shared_object = files[j];
        let treeitem = document.createElement("treeitem");
        let treerow = document.createElement("treerow");
        treeitem.appendChild(treerow);

        let cell = document.createElement("treecell");
        cell.setAttribute("label", host);
        treerow.appendChild(cell);

        cell = document.createElement("treecell");
        cell.setAttribute("label", files[j].name);
        treerow.appendChild(cell);

        cell = document.createElement("treecell");
        cell.setAttribute("label", files[j].size);
        treerow.appendChild(cell);

        cell = document.createElement("treecell");
        cell.setAttribute("label", files[j].date);
        treerow.appendChild(cell);

        treechildren.appendChild(treeitem);
      }
    }
  },

  show_cookies: function (session)
  {
    let names = ["host", "path", "name", "policy", "expires", "last accessed"];
    let tree = document.getElementById("query_tree");
    let treecols = this.get_element(tree, "treecols");
    let treechildren = this.get_element(tree, "treechildren");
    let database = window.database;

    this.strip_content(treecols);
    this.strip_content(treechildren);
    this.create_columns(treecols, names);

    let cm = Components.classes['@mozilla.org/cookiemanager;1'].
      getService(Components.interfaces.nsICookieManager2);

    let cookies = cm.enumerator;
    let results = [];

    while (cookies.hasMoreElements())
    {
      let cookie = cookies.getNext();
      cookie.QueryInterface(Components.interfaces.nsICookie2);

      //if (cookie)
      //  alert(cookie.host);

      if (cookie.isSession)
      {
        if (session)
          results.push(cookie);
      }
      else if (!session)
        results.push(cookie);
    }

    for (let i = 0; i < results.length; ++i)
    {
      cookie = results[i];
      var treeitem = document.createElement("treeitem");
      var treerow = document.createElement("treerow");
      treeitem.appendChild(treerow);

      let host = cookie.host;

      if (host.charAt(0) == '.')
        host = '*' + host;

      let cell = document.createElement("treecell");
      cell.setAttribute("label", host);
      treerow.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label", cookie.path);
      treerow.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label", cookie.name);
      treerow.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label", cookie.policy);
      treerow.appendChild(cell);

      // expires is in seconds since epoch
      let date = new Date((cookie.expires)*1000);

      cell = document.createElement("treecell");
      cell.setAttribute("label", database.iso8061(date));
      treerow.appendChild(cell);

      // last accessed is in microseconds since epoch
      date = new Date((cookie.lastAccessed)/1000);

      cell = document.createElement("treecell");
      cell.setAttribute("label", database.iso8061(date));
      treerow.appendChild(cell);

      treechildren.appendChild(treeitem);
    }
  },

  show_cookies_for_host: function (host)
  {
    let domain = this.base_domain(host);
    let names = ["host", "path", "name", "session", "secure", "P3P policy", 
                 "P3P status", "expires", "created", "last accessed"];
    let tree = document.getElementById("query_tree");
    let treecols = this.get_element(tree, "treecols");
    let treechildren = this.get_element(tree, "treechildren");
    let database = window.database;
    let date;

    this.strip_content(treecols);
    this.strip_content(treechildren);
    this.create_columns(treecols, names);

    let cm = Components.classes['@mozilla.org/cookiemanager;1'].
      getService(Components.interfaces.nsICookieManager2);

    let cookies = cm.enumerator;
    let results = [];

    while (cookies.hasMoreElements())
    {
      let cookie = cookies.getNext();
      cookie.QueryInterface(Components.interfaces.nsICookie2);

      let len = cookie.host.length;
      let i = cookie.host.indexOf(domain);

      // does host end with domain?
      if (i >= 0 && len == i + domain.length)
        results.push(cookie);
    }

    for (let i = 0; i < results.length; ++i)
    {
      let cookie = results[i];
      let treeitem = document.createElement("treeitem");
      let treerow = document.createElement("treerow");
      treeitem.appendChild(treerow);

      let host = cookie.host;

      if (host.charAt(0) == '.')
        host = '*' + host;

      let cell = document.createElement("treecell");
      cell.setAttribute("label", host);
      treerow.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label", cookie.path);
      treerow.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label", cookie.name);
      treerow.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label", cookie.isSession);
      treerow.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label", cookie.isSecure);
      treerow.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label", cookie.policy);
      treerow.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label", cookie.status);
      treerow.appendChild(cell);

      cell = document.createElement("treecell");

      // expires is in seconds since epoch
      if (cookie.isSession)
        cell.setAttribute("label", "");
      else
      {
        let date = new Date((cookie.expires)*1000);
        cell.setAttribute("label", database.iso8061(date));
      }

      treerow.appendChild(cell);

      // last accessed is in microseconds since epoch
      let date = new Date((cookie.creationTime)/1000);

      cell = document.createElement("treecell");
      cell.setAttribute("label", database.iso8061(date));
      treerow.appendChild(cell);

      // last accessed is in microseconds since epoch
      date = new Date((cookie.lastAccessed)/1000);

      cell = document.createElement("treecell");
      cell.setAttribute("label", database.iso8061(date));
      treerow.appendChild(cell);

      treechildren.appendChild(treeitem);
    }
  },

  strip_content: function (element)
  {
    while (element.firstChild)
      element.removeChild(element.firstChild);
  },

  create_element: function (tag, label) {
    let element = document.createElement(tag);
    element.setAttribute("label", label);
    return element;
  },

  get_element: function (parent, tag)
  {
     let node;

     for (node = parent.firstChild; node; node = node.nextSibling)
     {
       if (node.nodeType == 1 && node.nodeName == tag)
         break;
     }

     return node;
  },

  base_domain: function (url) // a string not an nsIURI
  {
    if (url.substr(0,6) == "about:")
      return "";

    let eTLDService = Components.classes["@mozilla.org/network/effective-tld-service;1"].
              getService(Components.interfaces.nsIEffectiveTLDService);

    let ioService = Components.classes["@mozilla.org/network/io-service;1"]  
                            .getService(Components.interfaces.nsIIOService); 
 
    try
    {
      if (! /^http.+/.test(url))
        url = "http://" + url + "/";

      let uri = ioService.newURI(url, null, null);
      return eTLDService.getBaseDomain(uri);
    }
    catch (e)
    {
      return url;
    }
  },

  // double up single quotes for SQL literals
  escape1: function (s)
  {
    if (s.indexOf("'") < 0)
      return s;

    let r = "";

    for (let i = 0; i < s.length; ++i)
    {
      let c = s.charAt(i);

      r += c;

      if (c == '\'')
        r += c;
    }

    return r;
  },

  // double ' _ and % for SQL LIKE literals
  escape2: function (s)
  {
    let r = "";

    for (let i = 0; i < s.length; ++i)
    {
      let c = s.charAt(i);

      r += c;

      if (c == '\'' || c == '_' || c == '%')
        r += c;
    }

    return r;
  },

  round: function (x, m)
  {
    x = Math.floor(x * m + 0.5);
    x = x / m;
    return x;
  }
};

window.addEventListener("load", function() { dashboard.on_load(); }, false); 
window.addEventListener("unload", function() { dashboard.on_unload(); }, false); 

