/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Dashboard.
 *
 * The Initial Developer of the Original Code is
 *   Dave Raggett.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */
dashboard_overlay.register_misc({
  purge_flash_cookies: function ()
  {
    let handler = 
    {
      hosts: {},
      count: 0,

      process_row: function (row)
      {
        var host = row.getResultByName("host");
        this.hosts[host] = host;
        this.count++;
      },

      // ready to clear flash shared objects
      finalize: function ()
      {
        dashboard_overlay.misc.delete_flash_cookies(this.hosts);
        this.hosts = null;
      }
    };

    // get a list of hosts for which we should clear flash cookies
    handler.misc = this;
    let database = dashboard_overlay.database;
    let sql = "SELECT host FROM site_info WHERE (prefs & 6)";

    database.select(sql, database.db, handler);
  },

  delete_flash_cookies: function (hosts)
  {
    if (!this.flash_dir)
      this.flash_dir = this.get_flash_dir();

    // append /Flash_Player/macromedia.com/support/flashplayer/sys
    let dir = this.flash_dir.clone();

    dir.append("Flash_Player");
    dir.append("macromedia.com");
    dir.append("support");
    dir.append("flashplayer");
    dir.append("sys");

    let folder = Components.classes["@mozilla.org/file/local;1"].  
                     createInstance(Components.interfaces.nsILocalFile);

    folder.initWithPath(dir.path);
    let flash_hosts = this.delete_hosts(folder, hosts);

    // append /Flash_Player/#SharedObjects
    dir = this.flash_dir.clone();
    dir.append("Flash_Player");
    dir.append("#SharedObjects");

    folder = Components.classes["@mozilla.org/file/local;1"].  
                     createInstance(Components.interfaces.nsILocalFile);

    folder.initWithPath(dir.path);

    // #SharedObjects contains named folder within which
    // reside the host specific folders we want to delete

    if (folder.exists() && folder.isDirectory() && folder.isReadable())
    {
       let content = folder.directoryEntries;

       while (content.hasMoreElements())
       {
         var file = content.getNext();
         file.QueryInterface(Components.interfaces.nsILocalFile);

         if (file.isDirectory())
           this.delete_hosts(file, hosts);
       }
    }
  },

  delete_hosts: function (folder, hosts)
  {
     let content = folder.directoryEntries;
     let file = content.hasMoreElements() ? content.getNext() : null;

     while (file)
     {
       let next = content.hasMoreElements() ? content.getNext() : null;

       file.QueryInterface(Components.interfaces.nsILocalFile);

       let name = file.leafName;

       if (name.charAt(0) == "#")
         name = name.substr(1);

       if (hosts[name])
         file.remove(true);

       file = next;
     }
  },

  scan_flash_storage: function ()
  {
    if (!this.flash_dir)
      this.flash_dir = this.get_flash_dir();

    // append /Flash_Player/macromedia.com/support/flashplayer/sys
    let dir = this.flash_dir.clone();

    dir.append("Flash_Player");
    dir.append("macromedia.com");
    dir.append("support");
    dir.append("flashplayer");
    dir.append("sys");

    let folder = Components.classes["@mozilla.org/file/local;1"].  
                     createInstance(Components.interfaces.nsILocalFile);

    folder.initWithPath(dir.path);
    let flash_hosts = this.collect_hosts(folder, []);

    // append /Flash_Player/#SharedObjects
    dir = this.flash_dir.clone();
    dir.append("Flash_Player");
    dir.append("#SharedObjects");

    folder = Components.classes["@mozilla.org/file/local;1"].  
                     createInstance(Components.interfaces.nsILocalFile);

    folder.initWithPath(dir.path);

    if (folder.exists() && folder.isDirectory() && folder.isReadable())
    {
       let content = folder.directoryEntries;
       let file;

       while (content.hasMoreElements())
       {
         file = content.getNext();
         file.QueryInterface(Components.interfaces.nsILocalFile);

         if (file.isDirectory())
           this.collect_hosts(file, flash_hosts);
       }
    }

    return flash_hosts;
  },

  collect_hosts: function (folder, hosts)
  {
    if (folder.exists() && folder.isDirectory() && folder.isReadable())
    {
       let content = folder.directoryEntries;
       let file, name, size, date, files;

       while (content.hasMoreElements())
       {
         file = content.getNext();
         file.QueryInterface(Components.interfaces.nsILocalFile);

         if (file.isDirectory())
         {
           name = file.leafName;

           if (name.charAt(0) == "#")
             name = name.substr(1);

           files = hosts[name];

           if (files)
             hosts[name] = hosts[name].concat(this.collect_files(file, []));
           else
             hosts[name] = this.collect_files(file, []);
         }
       }
    }

    return hosts;
  },

  // dir is an nsILocalFile enumeration, collection is
  // an array, scan nested directories for leaf files and
  // collect their names, sizes and dates last modified
  collect_files: function (dir, collection)
  {
    var content = dir.directoryEntries, file, lmd;

    while (content.hasMoreElements())
    {
      file = content.getNext();
      file.QueryInterface(Components.interfaces.nsILocalFile);

      if (file.isFile())
      {
        lmd = dashboard_overlay.database.iso8061(new Date(file.lastModifiedTime));
        collection.push({"name": file.leafName,
                         "size": file.fileSize,
                         "date": lmd});
      }
      else if (file.isDirectory())
        collection = this.collect_files(file, collection);
    }
    return collection;
  },

  // this only looks at common locations
  // and doesn't search rarer alternatives
  get_flash_dir: function ()
  {
    let flash_dir = null;

    // Windows
    try 
    {
      flash_dir = this.get_root_dir("AppData");
      flash_dir.append("Roaming");
      flash_dir.append("Macromedia");

      if (flash_dir.exists() && flash_dir.isDirectory())
        return flash_dir;
    }
    catch (e)
    {
      // alert("couldn't find windows flash dir ");
    }

    try
    {
      flash_dir = this.get_root_dir("AppData");
      flash_dir.append("Macromedia");

      if (flash_dir.exists() && flash_dir.isDirectory())
        return flash_dir;
    }
    catch (e)
    {
      // alert("couldn't find windows flash dir ");
    }

    // Apple
    try
    {
      flash_dir = this.get_root_dir("ULibDir");
      flash_dir.append("Preferences");
      flash_dir.append("Macromedia");

      if (flash_dir.exists() && flash_dir.isDirectory())
        return flash_dir;
    }
    catch (e)
    {
      // alert("couldn't find mac1 flash dir ");
    }

    try
    {
      flash_dir = this.get_root_dir("UsrPrfs");
      flash_dir.append("Macromedia");

      if (flash_dir.exists() && flash_dir.isDirectory())
        return flash_dir;
    }
    catch (e)
    {
      // alert("couldn't find mac2 flash dir ");
    }

    // Linux
    try
    {
      flash_dir = this.get_root_dir("Home");
      flash_dir.append(".macromedia");

      if (flash_dir.exists() && flash_dir.isDirectory())
        return flash_dir;
    }
    catch (e)
    {
      // alert("couldn't find linux flash dir ");
    }

    alert("couldn't find flash storage directory");
    return null;
  },

  get_root_dir: function (folder)
  {
    return Components.classes["@mozilla.org/file/directory_service;1"].
             getService(Components.interfaces.nsIProperties).
             get(folder, Components.interfaces.nsILocalFile);
  },

  html5_ping: function (browser)
  {
    let count = 0;
    let doc = browser.contentDocument;
    let links = doc.getElementsByTagName("a");
    let areas = doc.getElementsByTagName("a");
    let status = browser.dashboard_status;

    for (let i = 0; i < links.length; ++i)
    {
      if (links[i].getAttribute("ping"))
        ++count;
    }

    for (let i = 0; i < areas.length; ++i)
    {
      if (areas[i].getAttribute("ping"))
        ++count;
    }

    status.html5_pings = count;
  },

  // *** TO DO add check for compact policy in HTTP
  // responses as part of HTTP observer code ***
  get_p3p_policy: function (host, url, handler)
  {
    let req = new XMLHttpRequest();

    if (!url)
      url = "http://"+host+"/w3c/p3p.xml";

    req.open("GET", url, true);
    req.onreadystatechange = function (evt)
    {

      if (req.readyState == 4)
      {
        if (req.status == 200)
          handler(host, req);
        else if (req.status == 404)  // look for P3P header
        {
          let p3p = req.getResponseHeader("P3P");

          if (p3p)
          {
            let i = p3p.indexOf("policyref=");

            if (i >= 0)
            {
              let c = p3p.charAt(10);
              p3p = p3p.substr(11);
              i = p3p.indexOf(c);
              p3p = p3p.substring(0, i);
              return dashboard_overlay.misc.get_p3p_policy(host, p3p, handler);
            }
          }

          handler(host, null);
        }
        else
          handler(host, null);
      }
    };

    req.send(null);
  },

  // 3rd parties covers direct and indirect resources
  // p3p only covers current site ditto for geo, ping, dom storage
  // cookies covers current site and 3rd parties
  rate_site: function (browser)
  {
    let uri = browser.currentURI;
    let status = browser.dashboard_status;

    // status is null for about:* pages
    let host = (status ? status.page_host : null);
    console.log("misc.rate_site: " + host);
    
    if (host)
    {
      // synchronous functions
      this.html5_ping(browser);
      this.spot_web_bugs(browser);

      // prepare continuation, count is number of processes 
      // before calling misc.update_dashboard(browser);
      let counter = { "count": 4 };

      // asynchronous involving db access
      this.query_3rd_parties(browser, host, counter);
      this.query_p3p(browser, host, counter);
      this.query_geo(browser, host, counter);
      this.query_dom_storage(browser, counter);
    }
    else // treat non-http pages as cool
    {
      let primelife_button = document.getElementById("dashboard-toolbar-button");
      primelife_button.style.listStyleImage = 
             'url("chrome://dashboard-common/skin/glasses-cool.png")';

      this.update_dashboard(browser);
    }
  },

  // update dashboard current site user interface
  // introduce delay to allow change in browser to take effect
  update_dashboard: function (browser)
  {
    setTimeout(function ()
    {
      dashboard_overlay.misc.check_rating(browser);

      let status = browser.dashboard_status;
      let context = dashboard_overlay.context;
      let dialog = context.dashboard_dialog;
      console.log("update_dashboard: " + status.page_host);

      // check if this is the current browser to avoid changing dashboard if it's not
      if (dialog && dialog.dashboard && browser == dashboard_overlay.current_browser())
      {
        dialog.dashboard_overlay = dashboard_overlay;
        dialog.dashboard.update_current_site(status);
      }
    }, 10);
  },

  check_rating: function (browser)
  {
    let level = 0;
    let status = browser.dashboard_status;
    let host = status.page_host;

    if (status.ext_3rd_party_lasting_cookies || status.ext_3rd_party_flash_cookies)
      level = 2;
    else if ((!status.p3p) &&
       (status.lasting_cookies || status.flash_cookies || status.ext_3rd_parties))
      level = 1;

    dashboard_overlay.privacy_level = level;
  
    let primelife_button = document.getElementById("dashboard-toolbar-button");

    if (level == 2)
      primelife_button.style.listStyleImage =
                   'url("chrome://dashboard-common/skin/mad-tongue.png")';
    else if (level == 1)
      primelife_button.style.listStyleImage = 
                   'url("chrome://dashboard-common/skin/disappointed.png")';
    else
      primelife_button.style.listStyleImage =
                   'url("chrome://dashboard-common/skin/glasses-cool.png")';

    if (level > 0 && status.visited < 2 && !status.notified)
    {
      status.notified = true;
      dashboard_overlay.show_notification_bar(host, level);
    }

    let singleton_window =
        Components.classes["@mozilla.org/appshell/appShellService;1"].
          getService(Components.interfaces.nsIAppShellService).hiddenDOMWindow;

    singleton_window.dashboard_icon = primelife_button.style.listStyleImage;
  },

  // query for list of 3rd parties for this site, assumes that
  // database.log_3rd_parties(status); has already taken effect
  query_3rd_parties: function (browser, host, counter)
  {
    let database = dashboard_overlay.database;
    let query = "SELECT DISTINCT third_party, offsite FROM  parties " +
                "WHERE page_host = " + database.quote(host);

    let handler = 
    {
      parties: [],
      offsite: [],

      process_row: function (row)
      {
        this.parties.push(row.getResultByName("third_party"));
        this.offsite.push(row.getResultByName("offsite"));        
      },

      finalize: function () 
      {
        dashboard_overlay.misc.rate_cookies(browser, this.parties, this.offsite);

        if (--counter.count <= 0)
          dashboard_overlay.misc.update_dashboard(browser);
      }
    };

    database.select(query, database.db, handler);
  },

  query_dom_storage: function (browser, counter)
  {
    let database = dashboard_overlay.database;
    let status = browser.dashboard_status;
    let uri = browser.currentURI;
    let port = uri.port;

    if (port < 0)
    {
      if (uri.scheme == "https")
        port = 443;
      else
        port = "80";
    }

    let id = uri.host.split("").reverse().join("");
    id = id + ".:" + uri.scheme + ":" + port;
    
    let query = "SELECT key FROM webappsstore2 WHERE " +
              "scope=" + database.quote(id);

    let handler =
    {
      keys: [],

      process_row: function (row)
      {
        this.keys.push(row.getResultByName("key"));
      },

      finalize: function ()
      {
        status.dom_storage = this.keys.length;

        if (--counter.count <= 0)
          dashboard_overlay.misc.update_dashboard(browser);
      }
    };

    database.select(query, database.dom_storage, handler);
  },

  // iterate through regular cookies counting different
  // kinds for first party and third parties
  // then do same check for flash supercookies
  // linear in *total* number of cookies for all hosts
  // it may be faster(?) to scan separately for cookies
  // for the host and for each 3rd party
  rate_cookies: function (browser, parties, offsite) 
  {
    let cm = Components.classes['@mozilla.org/cookiemanager;1'].
      getService(Components.interfaces.nsICookieManager2);

    let cookies = cm.enumerator;
    let lasting_cookies = [];
    let session_cookies = [];
    let int_3rd_lasting_cookies = [];
    let int_3rd_session_cookies = [];
    let ext_3rd_lasting_cookies = [];
    let ext_3rd_session_cookies = [];
    let status = browser.dashboard_status;
    let host = status.page_host;

    // iterate through cookiemanager's list of current cookies
    // and check against first and all third party sites, this
    // would require *many* database queries using SQLite
    while (cookies.hasMoreElements())
    {
      let cookie = cookies.getNext();
      cookie.QueryInterface(Components.interfaces.nsICookie2);

      // cookie matching web page host
      if (host.indexOf(cookie.host) >= 0)
      {
        if (cookie.isSession)
          session_cookies.push(cookie);
        else
          lasting_cookies.push(cookie);
      }
      else  // check for 3rd party cookies
      {
        // pump up the cost!
        for (let i in parties)
        {
          if (parties[i].indexOf(cookie.host) >= 0)
          {
            if (offsite[i] == 2)
            {
              if (cookie.isSession)
                ext_3rd_session_cookies.push(cookie);
              else
                ext_3rd_lasting_cookies.push(cookie);
            }
            else
            {
              if (cookie.isSession)
                int_3rd_session_cookies.push(cookie);
              else
                int_3rd_lasting_cookies.push(cookie);
            }
          }
        }
      }
    }

    let fso = [];
    let int_3rd_fso = [];
    let ext_3rd_fso = [];

    // now examine flash cookies
    let hosts = this.scan_flash_storage();

    for (let aHost in hosts)
    {
      let files = hosts[aHost];

      if (aHost == host && hosts[aHost])
        fso.push(files);
      else if (parties[aHost])
      {
        if (offset[aHost] == 2)
          ext_3rd_fso.push(files);
        else
          int_3rd_fso.push(files);
      }
    }

    let int_3rd_parties = 0;
    let ext_3rd_parties = 0;

    for (let i = 0; i < offsite.length; ++i)
    {
      if (offsite[i] == 2)
        ++ext_3rd_parties;
      else
        ++int_3rd_parties;
    }

    status.session_cookies = session_cookies.length;
    status.lasting_cookies = lasting_cookies.length;
    status.flash_cookies = fso.length;
    status.int_3rd_parties = int_3rd_parties;
    status.ext_3rd_parties = ext_3rd_parties;
    status.int_3rd_party_session_cookies = int_3rd_session_cookies.length;
    status.int_3rd_party_lasting_cookies = int_3rd_lasting_cookies.length;
    status.ext_3rd_party_session_cookies = ext_3rd_session_cookies.length;
    status.ext_3rd_party_lasting_cookies = ext_3rd_lasting_cookies.length;
    status.int_3rd_party_flash_cookies = int_3rd_fso.length;
    status.ext_3rd_party_flash_cookies = ext_3rd_fso.length;

    // get prefs and purge unwanted cookies
    let database = dashboard_overlay.database;
    let prefs = database.get_prefs(host, database.db);
    let cookie = null;
    console.log("rate_cookies: prefs = " + prefs);

    if (prefs & (1<<5))  // purge all lasting cookies
    {
      for (let i = 0; i < lasting_cookies.length; ++i)
      {
        cookie = lasting_cookies[i];
        cm.remove(cookie.host, cookie.name, cookie.path, false);
      }

      for (let i = 0; i < int_3rd_lasting_cookies.length; ++i)
      {
        cookie = int_3rd_lasting_cookies[i];
        cm.remove(cookie.host, cookie.name, cookie.path, false);
      }

      for (let i = 0; i < ext_3rd_lasting_cookies.length; ++i)
      {
        cookie = ext_3rd_lasting_cookies[i];
        cm.remove(cookie.host, cookie.name, cookie.path, false);
      }

      if (prefs & (1<<4))  // purge all 3rd party cookies
      {
        for (let i = 0; i < ext_3rd_session_cookies.length; ++i)
        {
          cookie = ext_3rd_session_cookies[i];
          cm.remove(cookie.host, cookie.name, cookie.path, false);
        }
      }
    }
    else
    {
      if (prefs & (1<<4))  // purge all 3rd party cookies
      {
        for (let i = 0; i < ext_3rd_session_cookies.length; ++i)
        {
          cookie = ext_3rd_session_cookies[i];
          cm.remove(cookie.host, cookie.name, cookie.path, false);
        }

        for (let i = 0; i < ext_3rd_lasting_cookies.length; ++i)
        {
          cookie = ext_3rd_lasting_cookies[i];
          cm.remove(cookie.host, cookie.name, cookie.path, false);
        }
      }
    }
  },

  query_p3p: function (browser, host, counter)
  {
    let database = dashboard_overlay.database;
    let query = "SELECT p3p FROM site_info WHERE host = " + database.quote(host);
    let handler =
    {
      data: [],

      process_row: function (row)
      {
        this.data.push(row.getResultByName("p3p"));
      },

      finalize: function ()
      {
        dashboard_overlay.misc.rate_p3p(browser, this.data);

        if (--counter.count <= 0)
          dashboard_overlay.misc.update_dashboard(browser);
      }
    };

    database.select(query, database.db, handler);
  },

  rate_p3p: function (browser, data)
  {
    browser.dashboard_status.p3p = data.length == 1 ? data[0] : 0;
    this.check_rating(browser);
  },

  query_geo: function (browser, host, counter)
  {
    let database = dashboard_overlay.database;
    let query = "SELECT permission FROM moz_hosts WHERE type = 'geo' AND host = "
                + database.quote(host);
    let handler =
    {
      browser: browser,
      data: [],

      process_row: function (row)
      {
        this.data.push(row.getResultByName("permission"));
      },

      finalize: function ()
      {
        dashboard_overlay.misc.rate_geo(this.browser, this.data);

        if (--counter.count <= 0)
          dashboard_overlay.misc.update_dashboard(browser);
      }
    };

    database.select(query, database.permissions, handler);
  },

  rate_geo: function (browser, data)
  {
    let status = browser.dashboard_status;
    status.geo_permission = data.length == 1 ? data[0] : 0;
    this.check_rating(browser);
  },

  // look for web bugs as images with one or less pixel dimension
  // note visbility:hidden or display:none could be legitimate
  // TO DO - add check for suspicious urls with ids in them
  spot_web_bugs: function (browser)
  {
    let status = browser.dashboard_status;
    status.invisible_images = 0;
    let images = browser.contentDocument.getElementsByTagName("img");

    for (let i = 0; i < images.length; ++i)
    {
      if (this.spot_img_web_bug(images[i]))
        ++status.invisible_images;
    }
  },

  spot_img_web_bug: function (image)
  {
    let bug = false;

    if (image.width < 2 || image.height < 2)
      bug = true;

    let size = this.get_computed_style(image, "width");

    if (parseInt(size) < 2)
      bug = true;

    let size = this.get_computed_style(image, "width");

    if (parseInt(size) < 2)
      bug = true;

    if (this.get_computed_style(image, "display") == "none")
      bug = true;

    if (this.get_computed_style(image, "visibility") == "hidden")
      bug = true;

    return bug;
  },

  get_computed_style: function(elem, style)
  {
    let computed_style;

    if (typeof elem.currentStyle != 'undefined')
      computed_style = elem.currentStyle;
    else
      computed_style = document.defaultView.getComputedStyle(elem, null);

    return computed_style[style];
  }
});
