/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is dashboard_overlay.
 *
 * The Initial Developer of the Original Code is
 *   Dave Raggett.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your vdashboardersion of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

// these shortcuts are already defined for us :)
// but fail for policy object after chrome reload
//  const Cc = Components.classes;
//  const Ci = Components.interfaces;
//  const Cr = Components.results;
//  const Cu = Components.utils;

var console = {
  service: null,
  log: function(msg)
  {
    if (!console.service)
      console.service = Components.classes["@mozilla.org/consoleservice;1"]
                     .getService(Components.interfaces.nsIConsoleService);
    console.service.logStringMessage(msg);
  }
};

var dashboard_overlay =
{
  // register anonymous object as our observer module
  // which deals with page, http and tab events
  register_observer: function (module)
  {
    dashboard_overlay.observer = module;
  },

  // register anonymous object as our database module
  // which deals with sqlite database support
  register_database: function (module)
  {
    dashboard_overlay.database = module;
  },

  // register anonymous object as our misc module
  // which deals with flash, dom storage etc.
  register_misc: function (module)
  {
    dashboard_overlay.misc = module;
  },

  // register anonymous object as our share module
  // which deals with uploading site information
  // every few days for pooling information
  register_share: function (module)
  {
    dashboard_overlay.share = module;
  },

  on_start: function()
  {
    // called when firefox opens new browser window
    console.log("dashboard starting up for new window");
    window.dashboard_overlay = this;

    this.install_toolbar_icon();
    this.find_version();

    this.strings = document.getElementById("dashboard-strings");

    // There can only be one hidden DOM window and
    // we use it to avoid registering duplicate observers
    let singleton_window =
        Components.classes["@mozilla.org/appshell/appShellService;1"].
          getService(Components.interfaces.nsIAppShellService).hiddenDOMWindow;

    let container = gBrowser.tabContainer;

    container.addEventListener("TabSelect", 
      function(e) {dashboard_overlay.tab_selected(e);}, false);

    window.addEventListener("click",
      function(e) {dashboard_overlay.on_window_click(e);}, false);

    // extra work if our window is the first window
    if (!singleton_window.privacy_dashboard_context)
    {
      let context = {};
      context.window_count = 0;
      context.last_page = null;
      context.preferences = {}; // cache of preferences
      context.pages = {}; // set of currently loading pages

      // long name to avoid collisions with other extensions
      singleton_window.privacy_dashboard_context = context;
      singleton_window.dashboard_browser_window = window;
    }

    this.context = singleton_window.privacy_dashboard_context;   
    this.strings = document.getElementById("dashboard-strings");

    this.database.misc = this.misc;  // used by dashboard to access misc
    this.database.start_up(this.context); // set up sqlite
    this.observer.start_up(this.context); // set up http observers
    this.share.start_up(this.context); // initialize share module

    // increment window count *after* starting database/observers
    ++this.context.window_count;

    // set up handlers for web page load and unload events
    // DOMContentLoaded is fired before image content has all
    // been loaded, i.e. before "loaded" event is fired
    let appcontent = document.getElementById("appcontent");   // browser?

    if(appcontent)
    {
      appcontent.addEventListener("DOMContentLoaded",
        function(e)
        {
          dashboard_overlay.on_page_load(e);
        }, true);

      appcontent.addEventListener("unload",
        function(e)
        {
          dashboard_overlay.on_page_unload(e);
        }, true);

      appcontent.addEventListener("submit",
        function(e)
        {
          dashboard_overlay.on_form_submit(e);
        }, true);
    }

    this.misc.purge_flash_cookies();

    // dashbot support
    this.site_index = 0;

    if (typeof(top_sites) != "undefined")  // see top-sites.js
    {
      let ioService = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);

      var referrer = ioService.newURI("about:blank", null, null);
      console.log("starting visiting");
      setTimeout(function ()
      {
        dashboard_overlay.visit_site(referrer);
      }, 4000);
    }
  },

  on_exit: function()
  {
    // called when firefox closes window
    let context = this.context;

    let container = gBrowser.tabContainer;
    container.removeEventListener("TabSelect", this.tab_selected, false);

    let dialog = context.dashboard_dialog;

    if (dialog)
      dialog.close();

    if (context.window_count)
      --context.window_count;

    if (context.window_count == 0)
    {
      this.observer.shut_down(context);
      this.database.shut_down(context);
    }
  },

  // look for first run, or an upgraded version
  find_version: function ()
  {
    // get new version from the extension's install.rdf
    console.log("checking for dashboard version");
    try  // assume Firefox 4
    {
      Components.utils.import("resource://gre/modules/AddonManager.jsm");
      AddonManager.getAddonByID("dashboard@dave.raggett",
                  function(addon) { dashboard_overlay.check_version(addon.version); });
    }
    catch (e) // fall back for Firefox 3.6 and earlier
    {
      let em = Components.classes["@mozilla.org/extensions/manager;1"]
                        .getService(Components.interfaces.nsIExtensionManager);
      let addon = em.getItemForID("dashboard@dave.raggett");
      this.check_version(addon.version);
    }
  },

  check_version: function (cur_ver)
  {
    // get previously installed version from preferences if any
    let prefs = Components.classes["@mozilla.org/preferences-service;1"]
                          .getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("dashboard@dave.raggett.");

    let prev_ver = null;

    try
    {
      prev_ver = prefs.getCharPref("version");
      console.log("previous version is " + prev_ver);
    }
    catch (e)
    {
      // nothing to do
    }

    if (!prev_ver)
    {
      // *** debug -- uncomment next line for release
      //prefs.setCharPref("version", cur_ver);

      // insert first run code here e.g. by visiting home page
      window.setTimeout(function ()
      {
        console.log("first installation using version " + cur_ver);
      }, 1000);
    }
    else if (prev_ver != curr_ver)
    {
      // let the user know about the new version e.g. by
      // visiting the page describing the new features/fixes
      window.setTimeout(function ()
      {
        console.log("upgrade from " + prev_ver + " to " + curr_ver);
      }, 1000);
    }
  },

  // visit top sites, one every 20 seconds
  visit_site: function (referrer)
  {
    let domain = top_sites[this.site_index++];

    console.log("*** visiting " + "http://www." + domain);
    gBrowser.selectedBrowser.loadURI("http://www." + domain, referrer);

    if (this.site_index < top_sites.length)
    {
      setTimeout(function ()
      {
        dashboard_overlay.visit_site(referrer);
      }, 60000);
    }
  },

  // invoked when user clicks on different tab
  tab_selected: function (e)
  {
    var browser = gBrowser.getBrowserForTab(e.target)
    let uri = browser.currentURI;
 
    // introduce delay in calling update on current site status
    // to complete logging of data associated with current page
    setTimeout(function ()
    {
      dashboard_overlay.misc.rate_site(browser);
    }, 200);
  },

  localize: function (key)
  {
    try
    {
      return dashboard_overlay.strings.getString(key);
    }
    catch (e)
    {
      alert("couldn't get localized string for " + key + "\n " + e);
      return key;
    }
  },

  // install dashboard button in navigation toolbar if not already there
  install_toolbar_icon: function ()
  {
    try
    {
      let firefoxnav = document.getElementById("nav-bar");
      let curSet = firefoxnav.currentSet;

      if (curSet.indexOf("dashboard-toolbar-button") == -1)
      {
        let set;

        // Place the button before the urlbar

        if (curSet.indexOf("urlbar-container") != -1)
          set = curSet.replace(/urlbar-container/, "dashboard-toolbar-button,urlbar-container");
        else  // at the end
          set = curSet + ",dashboard-toolbar-button";

        firefoxnav.setAttribute("currentset", set);
        firefoxnav.currentSet = set;
        document.persist("nav-bar", "currentset");

        // If you don't do the following call, funny things happen

        try 
        {
          BrowserToolboxCustomizeDone(true);
        }
        catch (e)
        { }
      }
    }
    catch(e)
    { }
  },

  sync_prefs: function (host, prefs)
  {
    if (typeof(host) != "undefined" && typeof(prefs) != "undefined")
    {
      this.context.preferences[host] = prefs;
      this.apply_prefs(prefs);
    }
  },

  host_prefs: function (host)
  {
    let prefs;  // initially undefined

    if (!host)
      return prefs;

    prefs = this.context.preferences[host];
    let database = this.database;

    if (typeof(prefs) == "undefined")
    {
      if (database)
      {
         var prefs = database.get_prefs(host, database.db);

         // check SQLite table row and field are both defined
         if (typeof(prefs) == "undefined" || prefs == null)
         {
           // no, so prefs have yet to be recorded

           if (typeof(this.default_prefs) == "undefined")
             prefs = this.default_prefs = database.get_prefs("default", database.db);

           // default hasn't been registered
           if (typeof(prefs) == "undefined")
             prefs = 0;
         }
      }

      this.sync_prefs(host, prefs);
    }

    // if preferences are simple adjust bit vector accordingly
    // to cater for carefree, thoughtful or paranoid defaults
    let simple = prefs & 3;

    if (simple < 3)
    {
      if (simple == 0)
        prefs = 0;  // carefree: don't block anything
      else if (simple == 1)
        prefs = (1 << 4)|(1<<9);  // thoughtful
      else  // paranoid, so block pretty much everything!
        prefs = 2 | (1<<4)|(1<<5)|(1<<6)|(1<<7)|(1<<8)|(1<<9)|(1<<10)|(1<<11)|(1<<12)|(1<<13);
    }

    return prefs;
  },

  // browser wide preference settings - this will cause problems
  // when multiple pages are loaded concurrently e.g when the
  // browser is restarting with multiple tabs, however, not much
  // we can do to resolve this without an improved capability API
  apply_prefs: function (prefs)
  {
    let browser_prefs = Components.classes["@mozilla.org/preferences-service;1"]
          .getService(Components.interfaces.nsIPrefService).getBranch("browser.");
    let http_prefs = Components.classes["@mozilla.org/preferences-service;1"]
          .getService(Components.interfaces.nsIPrefService).getBranch("network.http.");
    let storage_prefs = Components.classes["@mozilla.org/preferences-service;1"]
          .getService(Components.interfaces.nsIPrefService).getBranch("dom.storage.");
    let javascript_prefs = Components.classes["@mozilla.org/preferences-service;1"]
         .getService(Components.interfaces.nsIPrefService).getBranch("javascript.");
    let geo_prefs = Components.classes["@mozilla.org/preferences-service;1"]
         .getService(Components.interfaces.nsIPrefService).getBranch("geo.");

    if (typeof(prefs) != "undefined")
    {
      if (prefs < 3)
      {
        // default is carefree: don't block anything
        if (prefs == 1)
          prefs |= (1 << 4)|(1<<9);  // thoughtful
        else if (prefs == 2) // paranoid, so block pretty much everything!
          prefs |= (1<<4)|(1<<5)|(1<<6)|(1<<7)|(1<<8)|(1<<9)|(1<<10)|(1<<11)|(1<<12)|(1<<13);
      }

      // see assess:save_prefs for bit position
      let block_scripting = prefs & (1 << 7);
      javascript_prefs.setBoolPref("enabled", !block_scripting);

      let block_geo = prefs & (1 << 8);
      geo_prefs.setBoolPref("enabled", !block_geo);

      let disable_ping = prefs & (1 << 9);
      let disable_referrer = prefs & (1 << 10);
      let disable_storage = prefs & (1 << 11);
      storage_prefs.setBoolPref("enabled", !disable_storage);
    }
    else
    {
      console.log("prefs is undefined");
      javascript_prefs.setBoolPref("enabled", true);
      geo_prefs.setBoolPref("enabled", true);
      browser_prefs.setBoolPref("send_pings", false);  // Firefox default
      http_prefs.setIntPref("sendRefererHeader", 2);
      storage_prefs.setBoolPref("enabled", true);
    }
  },

  // open or close dashboard dialog window
  open: function (open)
  {
    let browser = this.current_browser();
    let status = browser.dashboard_status;

    let uri = browser.currentURI;
    let context = this.context;

    if (context.dashboard_dialog && !open)
    {
      context.dashboard_dialog.close();
      context.dashboard_dialog = null;
    }
    else if (!context.dashboard_dialog)
    {
      if (/^http.+/.test(uri.spec))
      {
        this.context.browser = browser;
        dashboard_overlay.misc.rate_site(browser);
      }

      var dialog = context.dashboard_dialog = 
         window.openDialog('chrome://dashboard/content/dashboard.xul',
                'dashboard', 'chrome,dialog=no', status, window);
      dialog.database = dashboard_overlay.database;
      dialog.dashboard_overlay = dashboard_overlay;
    }
  },

  clear_database: function (all)
  {
    let prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    .getService(Components.interfaces.nsIPromptService);

    let localize = dashboard_overlay.localize;
    let title = localize("overlay.clear_data.title");
    let text = all ? localize("overlay.clear_data.message")
                   : localize("overlay.compact_data.message");
    let save_text = localize("overlay.save_data.message");
    let save = {"value":true};

    if (prompts.confirmCheck(null, title, text, save_text, save))
    {
      dashboard_overlay.context.preferences = [];

      if (save.value)  // true or false
        database_overlay.share.share_and_clear(all);
      else // clear database right now
        dashboard_overlay.database.reset(all);
    }
  },

  show_notification_bar: function (host, level)
  {
    var localize = dashboard_overlay.localize;

    if (!host)
      host = dashboard_overlay.current_page.host;

    if (!level)
      level = dashboard_overlay.privacy_level;

    var label = localize("overlay.notification.label")+" "+host+"?";
    var nb = gBrowser.getNotificationBox()
    var n = nb.getNotificationWithValue('privacy-warning')

    if(n)
    {
      n.label = label;
    }
    else
    {
      var buttons = [
      {
        label: localize("overlay.notification.accept.label"),
        accessKey: localize("overlay.notification.accept.key"),
        popup: null,
        callback: function (bar)
        {
          var prefs = 0;
          var dbo = dashboard_overlay;
          dbo.sync_prefs(host, prefs);
          dbo.database.set_prefs(host, prefs);
          dbo.apply_prefs(prefs);
        }
      },
      {
        label: localize("overlay.notification.protect.label"),
        accessKey: localize("overlay.notification.protect.key"),
        popup: null,
        callback: function (bar)
        {
          var prefs = 2|(1<<4)|(1<<5)|(1<<6)|(1<<7)|(1<<8)|(1<<9)|(1<<10)|(1<<11);
          var dbo = dashboard_overlay;
          dbo.sync_prefs(host, prefs);
          dbo.database.set_prefs(host, prefs);
          dbo.apply_prefs(prefs);
        }
      },
      {
        label: localize("overlay.notification.more.label"),
        accessKey: localize("overlay.notification.more.key"),
        popup: null,
        callback: function (bar)
        {
          dashboard_overlay.open(true);
        }
      }];

      var icon;

      if (level == 2)
        icon = "chrome://dashboard-common/skin/mad-tongue.png";
      else if (level == 1)
        icon = "chrome://dashboard-common/skin/disappointed.png";
      else
        icon = "chrome://dashboard-common/skin/glasses-cool.png";

      const priority = nb.PRIORITY_WARNING_MEDIUM;
      nb.appendNotification(label, 'privacy-warning',
                         icon, priority, buttons);
    }
  },

  on_form_submit: function (e)
  {
    //alert("form submit");
  },

  // this is used to ensure that the dashboard is updated
  // when clicking on a different browser window
  on_window_click: function (aEvent)
  {
    var singleton_window =
        Components.classes["@mozilla.org/appshell/appShellService;1"].
          getService(Components.interfaces.nsIAppShellService).hiddenDOMWindow;

    if (singleton_window.dashboard_browser_window != window)
    {
     // if (gBrowser.contentDocument.documentURIObject.scheme == "about")
     //   alert("clicked on blank window");

      var browser = this.current_browser();

      // introduce delay in calling update on current site status
      // to complete logging of data associated with current page
      setTimeout(function ()
      {
        dashboard_overlay.misc.rate_site(browser);
      }, 300);

      singleton_window.dashboard_browser_window = window;
    }
  },


  // further work needed to deal with iframes, for now ignore them
  on_page_load: function(aEvent)
  {
    let doc = aEvent.originalTarget;
    console.log("on page load event - " + doc.location.href);

    if (doc instanceof HTMLDocument)
    {
      let win = doc.defaultView;

      if (win.frameElement)
      {
        // find root document, needs further work!
        win = win.top;
        return;
      }

      // find browser for this document
      var browser = this.find_browser_from_doc(doc);
      var status = browser.dashboard_status;

      // status is null for about:* pages
      var host = (status ? status.page_host : null);

      if (status)
      {
        status.loaded = true;

        // some page redirects aren't caught by observer
        // perhaps we can do without the observer check?
        if (doc.documentURIObject.spec != doc.location.href)
          alert("mismatch between documentURIObject and location!");

        if (status.uri.spec != doc.documentURIObject.spec)
        {
          status.old_uri = status.uri;
          status.old_host = status.page_host;
          status.uri = doc.documentURIObject;
          status.page_host = null;
          status.new_url = null;

          if (/^http.+/.test(status.uri.spec))
            status.page_host = status.uri.host;
        }
      }
      else if (/^http.+/.test(doc.location.protocol))
        alert("overlay:page_load missing status for " + doc.location.href);

      if (host)
      {
        // doc is document that triggered "onload" event  
        // use this event to log observed http data
        this.observer.dump();  // asynchronous local database operation

        let uri = browser.currentURI;

        // check if the user has visited this site before
        this.database.mark_visited(browser);

        // firefox raises exception on accessing location.host
        // for URLs like about:home, so we first check protocol
        if (/^http.+/.test(doc.location.protocol))
        {
          this.misc.get_p3p_policy(doc.location.host, null,
          function (host, req)
          {
            dashboard_overlay.database.p3p_log(host, req);
          });
        }
        // page load may be handled before last http request notify
        // this is very surprising, and discovered by timing the events
        setTimeout(function ()
        {
          dashboard_overlay.database.log_3rd_parties(status);
        }, 200);
  
        // and a little later still log site rating to database
        setTimeout(function ()
        {
          dashboard_overlay.database.update_site_info(host, status);
          
        }, 2000);
      }
  
      // introduce delay in calling update on current site status
      // to complete logging of data associated with current page
      // and allow for web page scripts doing stuff on page load
      setTimeout(function ()
      {
        dashboard_overlay.misc.rate_site(browser);
      }, 500);
    }
  },
  
  // clear tab's dashboard status
  on_page_unload: function(aEvent)
  {  
    var doc = aEvent.originalTarget;

    if (doc instanceof HTMLDocument)
    {
      let win = doc.defaultView;

      if (win.frameElement)
        return;

      console.log("on page unload event - " + doc.location.href);

      // find browser for this document
      var browser = this.find_browser_from_doc(doc);

      if (browser)
        browser.dashboard_status = null;
    } 
  },

  // base domain of nsIURI is used for distinguishing
  // internal and external third party websites
  base_domain: function (uri)
  {
    let base;

    try
    {
      var eTLDService = Components.classes["@mozilla.org/network/effective-tld-service;1"]
                  .getService(Components.interfaces.nsIEffectiveTLDService);
      base = eTLDService.getBaseDomain(uri);
    }
    catch (e)
    {
      base == "";
    }

    return base;
  },

  current_browser: function ()
  {
    return gBrowser.getBrowserForTab(gBrowser.selectedTab);
  },

  // find browser given a document - used on page load
  find_browser_from_doc: function (doc)
  {
    var n = gBrowser.browsers.length;  

    for (var i = 0; i < n; i++)
    {  
      var b = gBrowser.getBrowserAtIndex(i);

      if (b.contentDocument == doc)
        return b;
    }

    return null;
  },

  // convenience function to trim leading/trailing spaces
  trim: function (s)
  {
    let i, j;

    for (i = 0; i < s.length && s.charCodeAt(i) == 32; ++i);
    for (j = s.length-1; j > 0 && s.charCodeAt(j) == 32; --j);

    if (i <= j)
      return s.substr(i, j-i+1);

    // otherwise leave string as is
    return s;
  }
};

// wait for window to load before accessing tabBrowser, see
// https://developer.mozilla.org/En/Code_snippets/Tabbed_browser

window.addEventListener("load", function() { dashboard_overlay.on_start(); }, false); 
window.addEventListener("unload", function() { dashboard_overlay.on_exit(); }, false); 


