/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Dashboard.
 *
 * The Initial Developer of the Original Code is
 *   Dave Raggett.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var dashboard_p3p =
{
  // *** TO DO add check for compact policy in HTTP
  // responses as part of HTTP observer code ***
  render_policy: function (uri, path, depth)
  {
    if (depth > 4)  // endless loop detector
      return alert("couldn't find correct policy for path: " + path);

    console.log("render_policy: uri = " + uri + " path = " + path);

    var req = new XMLHttpRequest();

    req.open("GET", uri, true);
    req.onreadystatechange = function (evt)
    {

      if (req.readyState == 4)
      {
        if (req.status == 200)
        {
          if (req.responseXML && req.responseXML.documentElement.nodeName != "parsererror")
            dashboard_p3p.find_applicable_policy(path, req.responseXML, depth);
          else
          {
/*
            if (req.channel.contentType == "text/html");
            {
              var parent_window = window.arguments ? window.arguments[1] : null;
              var browser = parent_window.gBrowser;
              browser.selectedTab =  browser.addTab(uri);
            }
*/
            console.log("trimming leading space before re-parse");
            // trim leading whitespace before XML declaration
            // so that we can view the policy for a common error
            let s = req.responseText;
            let i = 0;
            for (i = 0; i < s.length && s.charCodeAt(i) < 33; ++i);
            s = s.substr(i);

            let parser = new DOMParser();
            let dom = parser.parseFromString(s, "text/xml");

            if (dom.documentElement.nodeName != "parsererror")
              dashboard_p3p.find_applicable_policy(path, dom, depth);
            else
              alert("malformed P3P policy for url: " + uri +
                  "\nmedia type is " + req.channel.contentType);
          }
        }
        else if (req.status == 404)
        {
          let p3p = req.getResponseHeader("P3P");

          if (p3p)
          {
            let i = p3p.indexOf("policyref=");

            if (i >= 0)
            {
              let c = p3p.charAt(10);
              p3p = p3p.substr(11);
              i = p3p.indexOf(c);
              p3p = p3p.substring(0, i);
              return dashboard_p3p.render_policy(p3p, path, depth+1);
            }
          }

          alert("http status = " + req.status + " for url = " + uri);
        }
        else
          alert("http status = " + req.status + " for url = " + uri);
      }
    };
    req.send(null);
  },

// from XUL overlay, we can access req.channel.URI.resolve(rel_url)
// but not from regular web page script, so we use hack for now
// should match path to include and exclude statements, hack for now

  find_applicable_policy: function (path, doc, depth)
  {
    let serializer = new XMLSerializer();
    let xml = serializer.serializeToString(doc);
    //alert(xml);

    let uri = null, el, pattern;
    let root = doc.documentElement;
    console.log("p3p root element is " + root.nodeName);
    let includes = root.getElementsByTagName("INCLUDE");

    for (let i = 0; includes && i < includes.length; ++i)
    {
      el = includes[i];
      pattern = el.firstChild.nodeValue;  // should trim space - FIX ME

      if (pattern == "/*")
      {
        uri = includes[i].parentNode.getAttribute("about");
        break;
      }
    }

    if (!uri)
    {
      console.log("p3p: checking POLICY-REF element");
      let pr = root.getElementsByTagName("POLICY-REF");

      if (pr && pr.length > 0)
        uri = pr[0].getAttribute("about");
      else
        console.log("p3p: can't find any POLICY-REF elements");

      console.log("p3p policy-ref: " + uri);
    }

    if (uri)
    {
      let host = assess.host;
      let hash = uri.indexOf("#");

      if (hash == 0)
        dashboard_p3p.show_policy(path, root);
      else
      {
        if (hash >= 0)
          uri = uri.substr(0, hash);

        if (uri.substr(0,4) != "http")
        {
          if (uri.charAt(0) == "/")
            uri = "http://"+host+uri;
          else
            uri = "http://"+host+"/w3c/"+uri;
        }

        //alert("path for referenced policy: " + uri);
/*
        var base_uri = req.channel.URI;
        alert("base policy: " + base_uri.spec);

        var uri = base_uri.resolve(uri).spec;   
        alert("uri: " + uri);  
*/
        dashboard_p3p.render_policy(uri, path, depth+1);
      }
    }
    else
    {
      // otherwise render policy
      if (root.nodeName != "parsererror")
        dashboard_p3p.show_policy(path, root);
      else
        alert("malformed XML");
    }
  },

  show_policy: function (path, root)
  {
    let p3p = dashboard_p3p;
    console.log("show_policy: path = " + path);

    if (!p3p.description)
      p3p.description = document.getElementById("p3p_policy");

    let el, child;

    for (el = p3p.description; el.firstChild; el.removeChild(el.firstChild));

    el = document.createElementNS("http://www.w3.org/1999/xhtml", "em");
    p3p.description.appendChild(el);

    el.innerHTML = dashboard.localize("p3p_render.comment") + "<html:br />\n";

    let policies = root.getElementsByTagName("POLICY");
    let policy = policies ? policies[0] : null;

    if (!policy && root.nodeName == "POLICY")
      policy = root;

    if (!policy)
    {
      alert("couldn't find POLICY element");
      return;
    }

    let discuri = policy.getAttribute("discuri");
    let tab = document.getElementById("dashboard_current");
    tab.className = "policy";

    // use server's own rendering of policy
    if (false)
    {
       if (discuri)
       {
         location.href = discuri;
         return;
       }
    }

    // deal with contact info
    let contact = p3p.get_contact_group(policy);
    p3p.dump_contact(contact, discuri);

    // deal with access info
    let access = policy.getElementsByTagName("ACCESS");

    if (access && access.length > 0)
      access = access[0];

    // deal with dispute info
    // hmm, let's ignore this for now

    // deal with statements
    let statements = policy.getElementsByTagName("STATEMENT");
    for (let i = 0; i < statements.length; ++i)
      p3p.dump_statement(statements[i]);
  },

  get_contact_group: function (policy)
  {
    let ents = policy.getElementsByTagName("ENTITY");
  
    if (ents && ents.length > 0)
    {
      let dg = ents[0].getElementsByTagName("DATA-GROUP");
 
      if (dg && dg.length > 0)
        return dg[0];
    }

    return null;
  },

  dump_contact: function (contact, discuri)
  {
    let obj = {};
    console.log("p3p dump_contact: " + discuri);

    for (let data = contact.firstChild; data; data = data.nextSibling)
    {
      if (data.nodeType == 1)
      {
        let ref = data.getAttribute("ref");
        let value = data.firstChild.nodeValue;
        this.analyse_info(obj, ref, value);
      }
    }

    let div = document.createElementNS("http://www.w3.org/1999/xhtml", "div");
    //div.setAttribute("xmlns:html", "http://www.w3.org/1999/xhtml");
    let s = "";

    s += "<html:h2>"+obj.business.name+" P3P Privacy Policy</html:h2>\n";

    if (discuri)
    {
      s += "<html:p><html:em>See also site's <html:a id='shrp' title='" + discuri + "'>" +
           "human readable policy</html:a></html:em></html:p>\n";
    }

    s += "<html:p>\n";

    if (typeof(obj.business.contact_info.postal) != "undefined")
    {
      try
      {
        let info = obj.business.contact_info;

        s += info.postal.street + "<html:br />\n";
        s += info.postal.city + ", ";

        if (info.postal.stateprov)
        s += info.postal.stateprov + " ";
  
        if (info.postal.postalcode)
          s += info.postal.postalcode + "<html:br />\n";

        s += info.postal.country + "<html:br />\n";

        if (info.online.email)
        {
          s += "<html:br />email: \n";
          s += info.online.email;
        }

        if (info.telecom && info.telecom.telephone)
        {
          s += "<html:br />phone: \n";

          if (info.telecom.telephone.intcode)
          {
            s += "+"  + info.telecom.telephone.intcode;
            s += "."  + info.telecom.telephone.loccode;
            s += "."  + info.telecom.telephone.number;
          }
          else if (info.telecom.telephone.number)
          {
            if (info.telecom.telephone.number.substr(0,2) == "00")
              s += "+" + info.telecom.telephone.substr(2);
            else
              s += info.telecom.telephone.number;
          }
          else if (typeof (info.telecom.telephone) == "string")
          {
            if (info.telecom.telephone.substr(0,2) == "00")
              s += "+" + info.telecom.telephone.substr(2);
            else
            s += info.telecom.telephone;
          }
        }
      }
      catch (e)
      {
        console.log("p3p: couldn't format address for "  + obj.business.name);
      }
    }

    s += "\n</html:p>\n";
    this.description.appendChild(div);
    div.innerHTML += s;

    // hypertext link in XUL dialog normally opens in that dialog
    // we override that behavior here to force load into new tab
    // on the parent browser window, and to then select that tab
    var parent_window = window.arguments ? window.arguments[1] : null;
    let link = document.getElementById("shrp");

    if (discuri)
    {
      link.addEventListener("click", function (e) {
         let browser = parent_window.gBrowser;
         browser.selectedTab =  browser.addTab(discuri);
         e.cancel = true;
         e.stopPropagation();
         e.preventDefault();
         return false;
      }, false);
    }
  },

  // map ref string into object tree
  analyse_info: function (obj, ref, value)
  {
    ref = ref.substr(1).split(".");

    let part = obj;
    let name;

    for (let i = 0; i < ref.length - 1; ++i)
    {
      name = ref[i];
      name = name.replace(/-/g, "_");

      if (!part[name])
        part[name] = {};

      part = part[name];
    }

    name = ref[ref.length - 1];

    if (!part[name])
      part[name] = value;
  },

  dump_statement: function (statement)
  {
    let div =  document.createElementNS("http://www.w3.org/1999/xhtml", "div");
    this.description.appendChild(div);

    let p =document.createElementNS("http://www.w3.org/1999/xhtml", "p");
    p.innerHTML = this.get_consequence(statement);
    div.appendChild(p);

    p = document.createElementNS("http://www.w3.org/1999/xhtml", "p");
    div.appendChild(p); 
    p.innerHTML = "The follow kinds of data are collected";

    let ul = document.createElementNS("http://www.w3.org/1999/xhtml", "ul");
    div.appendChild(ul); 

    let el = statement.getElementsByTagName("DATA-GROUP");

    if (el && el.length > 0)
    {
      let data = el[0].getElementsByTagName("DATA");
      let li;
      let desc;

      for (let i = 0; i < data.length; ++i)
      {
        let ref = data[i].getAttribute("ref").substr(1);
        let desc = this.get_description(ref);
        if (!desc) desc = ref;

        // special case as details are given separately
        if (ref == "dynamic.miscdata")
          continue;

        li = document.createElementNS("http://www.w3.org/1999/xhtml", "li");
        ul.appendChild(li);
        li.innerHTML = desc;
      }
    }

    p = document.createElementNS("http://www.w3.org/1999/xhtml", "p");
    p.innerHTML = "This data is used for";
    div.appendChild(p); 

    let ul = document.createElementNS("http://www.w3.org/1999/xhtml", "ul");
    ul.className = "long";
    div.appendChild(ul); 

    el = statement.getElementsByTagName("PURPOSE");

    if (el && el.length > 0)
    {
      for (el = el[0].firstChild; el; el = el.nextSibling)
      {
        if (el.nodeType != 1)
          continue;

        let purpose = this.purposes[el.nodeName];
        if (!purpose)
          break;

        let li = document.createElementNS("http://www.w3.org/1999/xhtml", "li");
        ul.appendChild(li);
        try 
        {
          li.innerHTML = purpose+this.opt_in_out(el);
        }
        catch (e) 
        {
          alert("bad: " + purpose+this.opt_in_out(el));
        }
      }
    }

    p = document.createElementNS("http://www.w3.org/1999/xhtml", "p");
    div.appendChild(p); 
    p.innerHTML = "This data is used by";

    let ul = document.createElementNS("http://www.w3.org/1999/xhtml", "ul");
    ul.className = "long";
    div.appendChild(ul); 

    el = statement.getElementsByTagName("RECIPIENT");

    if (el && el.length > 0)
    {
      for (el = el[0].firstChild; el; el = el.nextSibling)
      {
        if (el.nodeType != 1)
          continue;

        let recipient = this.recipients[el.nodeName];
        if (!recipient) purpose = el.nodeName;

        let li = document.createElementNS("http://www.w3.org/1999/xhtml", "li");
        ul.appendChild(li);
        li.innerHTML = recipient+this.opt_in_out(el)+
           this.recipient_description(el);
      }
    }

    p = document.createElementNS("http://www.w3.org/1999/xhtml", "p");
    p.innerHTML = "Data retention:";
    div.appendChild(p);

    let ul = document.createElementNS("http://www.w3.org/1999/xhtml", "ul");
    ul.className = "long";
    div.appendChild(ul); 

    el = statement.getElementsByTagName("RETENTION");

    if (el && el.length > 0)
    {
      for (el = el[0].firstChild; el; el = el.nextSibling)
      {
        if (el.nodeType != 1)
          continue;

        let retention = this.retention[el.nodeName];

        if (!retention)
        {
          purpose = el.nodeName;
          retention = this.retention["undefined"];
        }

        let li = document.createElementNS("http://www.w3.org/1999/xhtml", "li");
        ul.appendChild(li);
        li.innerHTML = retention;
      }
    }
  },

  opt_in_out: function (el)
  {
    let req = el.getAttribute("required");

    if (req == "opt-in")
      return " (opt in)";

    if (req == "opt-out")
      return " (opt out)";

    return "";
  },

  recipient_description: function (el)
  {
    el = el.firstElementChild;

    if (el)
      return ", but note that " + el.firstChild.nodeValue;

    return "";
  },

  get_consequence: function (statement)
  {
    let el = statement.getElementsByTagName("CONSEQUENCE");

    if (el && el.length > 0)
      return el[0].firstChild.nodeValue;

    return null;
  },

  purposes:
  {
    "current" :
      "Completion and Support of Activity For Which Data Was " +
      "Provided<html:span>: Information may be used by the service provider " +
      "to complete the activity for which it was provided, whether " +
      "a one-time activity such as returning the results from a Web " +
      "search, forwarding an email message, or placing an order; or " +
      "a recurring activity such as providing a subscription service, " +
      "or allowing access to an online address book or electronic " +
      "wallet.</html:span>",
    "admin" :
      "Web Site and System Administration<html:span>: Information may be used " +
      "for the technical support of the Web site and its computer " +
      "system. This would include processing computer account " +
      "information, information used in the course of securing and " +
      "maintaining the site, and verification of Web site activity " +
      "by the site or its agents.</html:span>",
    "develop" :
      "Research and Development<html:span>: Information may be used to " +
      "enhance, evaluate, or otherwise review the site, service, " +
      "product, or market. This does not include personal information " +
      "used to tailor or modify the content to the specific individual " +
      "nor information used to evaluate, target, profile or contact " +
      "the individual.</html:span>",
    "tailoring" :
      "One-time Tailoring<html:span>: Information may be used to tailor " +
      "or modify content or design of the site where the information " +
      "is used only for a single visit to the site and not used for " +
      "any kind of future customization. For example, an online store " +
      "might suggest other items a visitor may wish to purchase based " +
      "on the items he has already placed in his shopping basket.</html:span>",
    "pseudo-analysis" :
      "Pseudonymous Analysis<html:span>: Information may be used to create " +
      "or build a record of a particular individual or computer that " +
      "is tied to a pseudonymous identifier, without tying identified " +
      "data (such as name, address, phone number, or email address) " +
      "to the record. This profile will be used to determine the habits, " +
      "interests, or other characteristics of individuals for purpose " +
      "of research, analysis and reporting, but it will not be used to " +
      "attempt to identify specific individuals. For example, a marketer " +
      "may wish to understand the interests of visitors to different " +
      "portions of a Web site.</html:span>",
    "pseudo-decision" :
      "Pseudonymous Decision<html:span>: Information may be used to create " +
      "or build a record of a particular individual or computer that " +
      "is tied to a pseudonymous identifier, without tying identified " +
      "data (such as name, address, phone number, or email address) to " +
      "the record. This profile will be used to determine the habits, " +
      "interests, or other characteristics of individuals to make a " +
      "decision that directly affects that individual, but it will not " +
      "be used to attempt to identify specific individuals. For example, " +
      "a marketer may tailor or modify content displayed to the browser " +
      "based on pages viewed during previous visits.</html:span>",
    "individual-analysis" :
      "Individual Analysis<html:span>: Information may be used to determine " +
      "the habits, interests, or other characteristics of individuals " +
      "and combine it with identified data for the purpose of research, " +
      "analysis and reporting. For example, an online Web site for a " +
      "physical store may wish to analyze how online shoppers make " +
      "offline purchases.</html:span>",
    "individual-decision" :
      "Individual Decision<html:span>:  Information may be used to determine " +
      "the habits, interests, or other characteristics of individuals " +
      "and combine it with identified data to make a decision that " +
      "directly affects that individual.  For example, an online store " +
      "suggests items a visitor may wish to purchase based on items he " +
      "has purchased during previous visits to the Web site.</html:span>",
    "contact" :
      "Contacting Visitors for Marketing of Services or Products<html:span>: " +
      "Information may be used to contact the individual, through a " +
      "communications channel other than voice telephone, for the " +
      "promotion of a product or service. This includes notifying " +
      "visitors about updates to the Web site. This does not include a " +
      "direct reply to a question or comment or customer service for a " +
      "single transaction -- in those cases, <current/> would be used. " +
      "In addition, this does not include marketing via customized Web " +
      "content or banner advertisements embedded in sites the user is " +
      "visiting.</html:span>",
    "historical" :
      "Historical Preservation<html:span>: Information may be archived or " +
      "stored for the purpose of preserving social history as governed " +
      "by an existing law or policy. This law or policy MUST be " +
      "referenced in the <DISPUTES> element and MUST include a specific " +
      "definition of the type of qualified researcher who can access the " +
      "information, where this information will be stored and specifically " +
      "how this collection advances the preservation of history.</html:span>",
    "telemarketing" :
      "Contacting Visitors for Marketing of Services or Products Via " +
      "Telephone<html:span>: Information may be used to contact the individual " +
      "via a voice telephone call for promotion of a product or service. " +
      "This does not include a direct reply to a question or comment or " +
      "customer service for a single transaction.</html:span>",
    "other-purpose" : "Additional purposes."
  },

  recipients:
  {
    "ours" :
      "Ourselves and/or entities acting as our agents or entities for " +
      "whom we are acting as an agent<html:span>: An agent in this instance " +
      "is defined as a third party that processes data only on behalf " +
      "of the service provider for the completion of the stated purposes. " +
      "(e.g., the service provider and its printing bureau which prints " +
      "address labels and does nothing further with the information.)</html:span>",
    "delivery" :
      "Delivery services possibly following different practices<html:span>: " +
      "Legal entities performing delivery services that may use data for " +
      "purposes other than completion of the stated purpose. This should " +
      "also be used for delivery services whose data practices are " +
      "unknown.</html:span>",
    "same" :
      "Legal entities following our practices<html:span>: Legal entities who " +
      "use the data on their own behalf under equable practices. (e.g., " +
      "consider a service provider that grants the user access to collected " +
      "personal information, and also provides it to a partner who uses " +
      "it once but discards it. Since the recipient, who has otherwise " +
      "similar practices, cannot grant the user access to information " +
      "that it discarded, they are considered to have equable " +
      "practices.)</html:span>",
    "other-recipient" :
      "Legal entities following different practices<html:span>: Legal entities "+
      "that are constrained by and accountable to the original service " +
      "provider, but may use the data in a way not specified in the " +
      "service provider's practices (e.g., the service provider collects " +
      "data that is shared with a partner who may use it for other " +
      "purposes. However, it is in the service provider's interest to " +
      "ensure that the data is not used in a way that would be considered " +
      "abusive to the users' and its own interests.)</html:span>",
    "unrelated" :
      "Unrelated third parties<html:span>: Legal entities whose data usage " +
      "practices are not known by the original service provider.</html:span>",
    "public" :
      "Public fora<html:span>: Public fora such as bulletin boards, public " +
      "directories, or commercial CD-ROM directories.</html:span>"
  },

  retention:
  {
    "no-retention" :
      "Information is not retained for more than a brief period of time " +
      "necessary to make use of it during the course of a single online " +
      "interaction. <html:span>Information MUST be destroyed following this " +
      "interaction and MUST NOT be logged, archived, or otherwise stored. " +
      "This type of retention policy would apply, for example, to services " +
      "that keep no Web server logs, set cookies only for use during a " +
      "single session, or collect information to perform a search but do " +
      "not keep logs of searches performed.</html:span>",
    "stated-purpose" :
      "Information is retained to meet the stated purpose. <html:span>This " +
      "requires information to be discarded at the earliest time possible. " +
      "Sites MUST have a retention policy that establishes a destruction " +
      "time table. The retention policy MUST be included in or linked " +
      "from the site's human-readable privacy policy.</html:span>",
    "legal-requirement" :
      "As required by law or liability under applicable law<html:span>: " +
      "Information is retained to meet a stated purpose, but the " +
      "retention period is longer because of a legal requirement or " +
      "liability. For example, a law may allow consumers to dispute " +
      "transactions for a certain time period; therefore a business may " +
      "for liability reasons decide to maintain records of transactions, " +
      "or a law may affirmatively require a certain business to maintain " +
      "records for auditing or other soundness purposes. Sites MUST have " +
      "a retention policy that establishes a destruction time table. The " +
      "retention policy MUST be included in or linked from the site's " +
      "human-readable privacy policy.</html:span>",
    "business-practices" :
      "Determined by service provider's business practice<html:span>: " +
      "Information is retained under a service provider's stated " +
      "business practices. Sites MUST have a retention policy that " +
      "establishes a destruction time table. The retention policy MUST " +
      "be included in or linked from the site's human-readable privacy " +
      "policy.</html:span>",
    "indefinitely" :
      "Indefinitely<html:span>: Information is retained for an indeterminate " +
      "period of time. The absence of a retention policy would be " +
      "reflected under this option. Where the recipient is a public " +
      "fora, this is the appropriate retention policy.</html:span>",
    "undefined" :
      "<html:span>A site specific retention policy.</html:span>"
  },

  get_description: function (name)
  {
    let desc = this.data_description[name];
    if (!desc) desc = name;
    return desc;
  },

  data_description:
  {
    "user.name" : "user's name",
    "user.home-info" : "user's home contact information",
    "user.home-info.postal" : "user's address",
    "user.home-info.telecom" : "user's phone number",
    "user.home-info.online.email" : "user's email",
    "user.employer" : "user's employer",
    "dynamic.clickstream" : "information typically found in Web " +
      "server access logs, e.g.  the IP address or hostname of the user's " +
      "computer, the URI of the resource requested, the time the request " +
      "was made, the HTTP method used in the request, the size of the " +
      "response, and the HTTP status code in the response.",
    "dynamic.http" : "additional information carried in HTTP requests",
    "dynamic.http.useragent" : "additional information carried " + 
      "in HTTP user agent headers",
    "dynamic.clientevents" : "information about how the user " +
      "interacts with a web page, e.g. mouse movements and clicks",
    "dynamic.searchtext" :"search terms entered by the user",
    "dynamic.interactionrecord": "additional information on " +
      "user interaction, e.g. account transactions, etc.",
    "dynamic.cookies" : "the use of web page cookies for various " +
      "purposes, e.g. session identifiers and status information"
  }
};

