/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is dashboard_overlay.
 *
 * The Initial Developer of the Original Code is
 *   Dave Raggett.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your vdashboardersion of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

// #### FIX date hack on line 98 ####

dashboard_overlay.register_share({
  start_up: function (context)
  {
    if (context.window_count == 0)
    {
      // Get the "extensions.myext." branch
      var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                    .getService(Components.interfaces.nsIPrefService);
      prefs = prefs.getBranch("extensions.privacydashboard.");

      let sharing = false;

      // allow for undefined preference

      try {
        sharing = prefs.getBoolPref("share.enabled");
      } catch (e) {  }

      this.set_mode(sharing);
    }
  },

  set_mode: function (mode)
  {
    // Get the "extensions.myext." branch
    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                  .getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("extensions.privacydashboard.");

    prefs.setBoolPref("share.enabled", mode);

    if (!mode)
    {
      if (this.sharing)
      {
        try {
          clearInterval(this.interval_id);
        } catch (e) { }
      }

      this.sharing = false;
    }
    else
    {
      var self = this;
      self.sharing = true;

      // allow for very long lived browser sessions
      // test every hour as setInterval treats very
      // long delay e.g. 28 days as zero :(
      self.interval_id = setInterval(function ()
      {
        self.check_date();
      }, 60*60*1000);
    }
  },

  check_date: function ()
  {
    // Get the "extensions.myext." branch
    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                  .getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("extensions.privacydashboard.");

    var share_interval = prefs.getIntPref("share.interval"); // in days
    var last_shared = prefs.getCharPref("share.last"); // parsable date

    let now =  Date.now();

    if (last_shared == "")
      last_shared = now;
    else
      last_shared = Date.parse(last_shared);

    var self = this;
    let gap = Date.now() - last_shared;  // in milliseconds
    share_interval *= 24*60*60*1000;  // days to milliseconds

    var postbox = {};
    postbox.delivered = function (table)
    {
    };

    //if (gap > share_interval)
    {
      // wait 5 seconds and start upload
      setTimeout(function ()
      {
        self.share_data(prefs, last_shared, postbox);
      }, 1000);
    }
  },

  // called after user has activated database clear or compact
  // menu actions and wants to share the date before clearing.
  // a postbox object is used to ensure that the clear operation
  // only takes place when all three tables have been delivered
  share_and_clear: function (all)
  {
    // Get the "extensions.myext." branch
    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                  .getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("extensions.privacydashboard.");

    var last_shared = prefs.getCharPref("share.last"); // parsable date

    let now =  Date.now();

    if (last_shared == "")
      last_shared = now;
    else
      last_shared = Date.parse(last_shared);

    var postbox = {};
    postbox.table_count = 3; // number of tables to deliver
    postbox.delivered = function (table)
    {
      if (--this.table_count == 0)
        dashboard_overlay.database.reset(all);
    };
    
    this.share_data(prefs, last_shared, postbox);
  },

  // SQLite date format "2010-11-23 18:39:46"
  share_data: function (prefs, last, postbox)
  {
    postbox.to = prefs.getCharPref("share.with");

    if (!postbox.to)
      return;

    console.log("sharing data with " + postbox.to);
    prefs.setCharPref("share.last", Date() + "");

    let date = this.get_date(/*last*/ "23 November 2010 18:39:47");

    this.share_table("relations",
        ["parent", "child", "offsite"], date, postbox);
    this.share_table("parties",
        ["page_host", "third_party", "offsite"], date, postbox);
    this.share_table("site_info", 
        ["host",
         "visited",
         "session_cookies", 
         "lasting_cookies",
         "flash_cookies",
         "int_3rd_parties",
         "ext_3rd_parties",
         "int_3rd_party_session_cookies",
         "int_3rd_party_lasting_cookies",
         "int_3rd_party_flash_cookies",
         "ext_3rd_party_session_cookies",
         "ext_3rd_party_lasting_cookies",
         "ext_3rd_party_flash_cookies",
         "dom_storage",
         "html5_pings",
         "invisible_images",
         "suspicious_urls",
         "geo_permission",
         "p3p"], date, postbox);

  },

  share_table: function (table, col_names, date, postbox)
  {
    let sql = "SELECT " + this.column_list(col_names) + " FROM " + table +
              " WHERE time > '" + date +"'";
 
    var self = this;
    let database = dashboard_overlay.database;
    let handler = 
    {
      result: "",

      process_row: function (row)
      {
        let names = col_names;

        for (let i = 0; i < names.length; ++i)
        {
          let delim = "";
          let value = row.getResultByName(names[i]);

          if (typeof(value) == "string")
            delim = "'";

          this.result += (i ? "," : "") + delim + value + delim;
        }

        this.result += "\n";
      },

      finalize: function ()
      {
        self.deliver(table, this.result, postbox);
      }
    };

    database.select(sql, database.db, handler);
  },

  // include crc32 checksum as a very basic
  // protection against spoofed data uploads
  deliver: function (table, data, postbox)
  {
    var req = new XMLHttpRequest();
    var self = this;

    req.open("POST", postbox.to, true);
    req.setRequestHeader("Content-Type", "text/plain");
    req.setRequestHeader("X-Data-Check", this.adler32(data));
    req.setRequestHeader("Dashboard-Table", table);
    req.onreadystatechange = function (evt)
    {
      if (req.readyState == 4)
        postbox.delivered(table); // notify delivery
    }
    req.send(data);
  },

  column_list: function (list)
  {
    let names = "";

    for (let i = 0; i < list.length; ++i)
      names += (i ? ", " : "") + list[i];

    return names;
  },

  get_date: function (date)
  {
    date = new Date(date);
    return date.getFullYear() + "-" + (1+date.getMonth()) + "-" +
           date.getDate() + " " + date.getHours() + ":" +
           date.getMinutes() + ":" + date.getSeconds();
  },

  adler32: function (str)
  {
    let a = 1, b = 0;

    for (let i = 0; i < str.length; ++i)
    {
      a = (a + str.charCodeAt(i)) % 65521;
      b = (b + a) % 65521;
    }

    return a + b * 65536;
  }
});

