#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define NUM_FIELDS 7

typedef enum { JAVASCRIPT, C_LANGUAGE} Mode;

Mode mode = JAVASCRIPT;
int minimal = 0;

// parse number like 4,900,000,000
static long int read_number(char *number)
{
  char *p, *q;
  long int num = 0;

  p = q = number;

  while (*q)
  {
    if (*q == ',')
      ++q;
    else
      *p++ = *q++;
  }

  *p = '\0';

  sscanf(number, "%ld", &num);
  return num;
}

static char *trim_space(char *str)
{
  while (*str == ' ')
    ++str;

  char *p = str + strlen(str);

  while (p > str && *(--p) == ' ')
  {
    *p = '\0';
  }

  return str;
}

static void output_line(int line_num, char *line, int length, FILE *fout)
{
  long int rank;
  char *domain;
  char *category;
  long int visitors;
  double percent;
  long int hits;
  int ads;
  int i, j;

  char *field[NUM_FIELDS];

  field[0] = line;

  for (i = j = 1; i < length && j < NUM_FIELDS; ++i)
  {
     if (line[i] == '\t')
     {
       line[i] = '\0';
       field[j++] = line + i + 1;
     }
  }

  fprintf(fout, "%s", line_num > 1 ? ",\n" : "");

  sscanf(field[0], "%ld", &rank);
  domain = trim_space(field[1]);
  category = trim_space(field[2]);
  visitors = read_number(field[3]);
  sscanf(field[4], "%lg", &percent);
  hits = read_number(field[5]);
  ads = strcmp(field[6], "Yes") ? 0 : 1;

  if (mode == JAVASCRIPT)
  {
    if (minimal)
      fprintf(fout, "\"%s\"", domain);
    else
      fprintf(fout, "{\"rank\":%ld, \"domain\":\"%s\", \"category\":\"%s\", "
        "\"visitors\":%ld, \"percent\":%lg, \"hits\":%ld, \"ads\":%s}",
        rank, domain, category, visitors, percent, hits, (ads?"true":"false"));
  }
  else if (mode == C_LANGUAGE)
  {
    fprintf(fout, "{%ld, \"%s\", \"%s\", %ld, %lg, %ld, %s}",
      rank, domain, category, visitors, percent, hits, (ads?"true":"false"));
  }
}

static int read_line(int line_num, FILE *fin, FILE *fout)
{
  static char line[8192];
  char *p = line;
  
  while (!feof(fin))
  {
    int c = getc(fin);

    if (c == '\n')
      break;

    *p++ = c;
  }

  *p = '\0';

  if (p > line)
  {
    output_line(line_num, line, p - line, fout);
    return 1;
  }

  return 0;
}

int main(int argc, char *argv[])
{
  char *prog = argv[0];

  while (argc > 1 && *argv[1] == '-')
  {
    if (strcmp(argv[1], "-js") == 0)
      mode = JAVASCRIPT;
    else if (strcmp(argv[1], "-c") == 0)
      mode = C_LANGUAGE;
    else if (strcmp(argv[1], "--min") == 0)
      minimal = 1;
    else
    {
      fprintf(stderr, "%s [-js | -c ] [--min] [[srcfile] | [destfile]]\n"
      " a program to extract top site data to JavaScript or C\n", prog);

      return 1;
    }

    --argc;
    ++argv;    
  }

  char *in = (argc > 1 ? argv[1] : "top-sites.txt");
  char *out = (argc > 2 ? argv[2] :
              (mode == C_LANGUAGE ? "top-sites.c" : "top-sites.js"));


  FILE *fin = fopen(in, "r");

  if (!fin)
  {
    fprintf(stderr, "couldn't open %s for reading\n", in);
    return 1;
  }

  FILE *fout = fopen(out, "w+");

  if (!fout)
  {
    fprintf(stderr, "couldn't open %s for writing\n", out);
    return 1;
  }


  fprintf(fout, "// top 1000 sites according to google\n\n");

  if (mode == JAVASCRIPT)
    fprintf(fout, "top_sites = [\n");
  else if (mode == C_LANGUAGE)
  {
    fprintf(fout, "#ifndef __cplusplus\n\n"
       "#ifndef WS_BOOL\n"
       "#define WS_BOOL\n"
       "typedef enum { false, true } bool;\n"
       "#define null (void *)0\n"
       "#endif\n\n"
       "#endif\n\n");

    fprintf(fout, "typedef struct {\n"
       "  long int rank;\n"
       "  char *domain;\n"
       "  char *category;\n"
       "  long int visitors;\n"
       "  double percent;\n"
       "  long int hits;\n"
       "  bool ads;\n} SiteDesc;\n\n"
       "SiteDesc top_sites[1000] = {\n"
    );
  }

  int i = 0;

  while (read_line(++i, fin, fout));

  if (mode == JAVASCRIPT)
    fprintf(fout, "\n];\n");
  else if (mode == C_LANGUAGE)
    fprintf(fout, "\n};\n");

  fclose(fout);
  fclose(fin);
  return 0;
}
