/* query.js */
var dashboard = {
  on_load: function ()
  {
    this.query = document.getElementById("query");
    this.param = document.getElementById("param");
    this.param_caption = document.getElementById("param_caption");
    this.backward = document.getElementById("backward");
    this.forward = document.getElementById("forward");
    this.execute = document.getElementById("execute");
    this.site_info = document.getElementById("site_info");
    this.table = document.getElementById("table");
    this.undo = [];
    this.redo = 0;

    this.query.addEventListener("change", function(e)
    {
      dashboard.set_query(e);
    }, false);

    this.query.addEventListener("keydown", function (e)
    {
      var key = e ? e.which : window.event.keyCode;

      if (key == 13)
        dashboard.do_query();
    }, false);

    this.param.addEventListener("change", function(e)
    {
      dashboard.set_param(e);
    }, false);

    this.param.addEventListener("keydown", function (e)
    {
      var key = e ? e.which : window.event.keyCode;

      if (key == 13)
        dashboard.do_query(false);
    }, false);

    this.backward.addEventListener("click", function(e)
    {
      dashboard.go_backward(e);
    }, false);

    this.forward.addEventListener("click", function(e)
    {
      dashboard.go_forward(e);
    }, false);

    this.execute.addEventListener("click", function(e)
    {
      dashboard.do_query(false);
    }, false);

    this.site_info.addEventListener("click", function(e)
    {
      dashboard.get_site_info(false);
    }, false);

    this.table.addEventListener("click", function(e)
    {
      dashboard.table_click(e);
    }, false);
  },

  on_unload: function ()
  {
  },

  get_caption: function ()
  {
    var caption = null;

    switch (this.query.value)
    {
      case "9":
      case "10":
      case "11":
      case "13":
      case "99":
        caption = "2. Website domain";
        break;

      default:
        caption = null;
    }

    if (caption)
    {
      this.param_caption.innerHTML = caption;
      this.param.disabled = false;
      this.param.className = "";
      this.param_caption.className = "";
      this.param.focus();
    }
    else
    {
      this.param_caption.innerHTML = "2. Query parameter";
      this.param.disabled = true;
      this.param.className = "inactive";
      this.param_caption.className = "inactive";
    }

    if (this.param.value && this.param.value != "")
      this.site_info.disabled = false;
    else
      this.site_info.disabled = true;

    return caption;
  },

  set_query: function (e)
  {
    if (!this.get_caption())
      this.do_query();
  },

  set_param: function (e)
  {
  },

  table_click: function (e)
  {
    if (e.target.nodeName.toLowerCase() == "td")
    {
      var text = e.target.innerHTML;

      if (text)
      {
        this.param.value = text;
        this.site_info.disabled = false;
        this.site_info.className = "";
      }
    }
  },

  go_backward: function (e)
  {
    if (this.undo.length - this.redo > 0)
    {
      this.forward.disabled = false;
      ++this.redo;

      var state = null;

      if (this.undo.length - this.redo > 0)
        state = this.undo[this.undo.length - this.redo - 1];

      this.sync_state(state);

      if (this.undo.length - this.redo <= 0)
        this.backward.disabled = true;
    }
  },

  go_forward: function (e)
  {
    if (this.redo > 0)
    {
      this.backward.disabled = false;

      state = this.undo[this.undo.length - this.redo];
      this.sync_state(state);

      if (--this.redo == 0)
        this.forward.disabled = true;
    }
  },

  sync_state: function (state)
  {
    if (state == null)
    {
      this.clear_table();
      this.query.value = "0";
      this.param.value = "";
      this.get_caption();
    }
    else
    {
      this.query.value = state.query;
      this.param.value = state.param;
      this.get_caption();
      this.do_query(true);
    }
  },

  push_undo: function ()
  {
    if (this.redo)
    {
      this.undo.splice(this.undo.length - this.redo);
      this.redo = 0;
      this.forward.disabled = true;
    }

    this.backward.disabled = false;
    this.undo.push({"query":this.query.value,"param":this.param.value});
  },

  do_query: function (redo)
  {
    if (this.query.value == 99)
      return this.get_site_info(redo);

    if (!redo)
      this.push_undo();

    if (!this.param.disabled && this.param.value == "")
      alert("please provide value for search parameter");
    else
    {
      var req = new XMLHttpRequest();
      req.open("POST", "query.php?query="+this.query.value+
                       "&param="+this.param.value, true);
      req.onreadystatechange = function (evt) 
      {
        if (req.readyState == 4)
        {
          if (req.status == 200)
            dashboard.display_results(req);
        }
      };

      req.send(null);
    }
  },

  // data is returned as sequence of records terminated by \n
  // values separated by \t, and first record as column names
  display_results: function (req)
  {
    var table = document.getElementById("table");
    var data = req.responseText;
    
    var records = data.split("\n");
    var i, j, record, thead, tbody, row, cell;

    // clear table's content
    this.clear_table();

    // first deal with column names
    thead = document.createElement("thead");
    row = document.createElement("tr");
    thead.appendChild(row);
    record = records[0].split("\t");

    for (i = 0; i < record.length; ++i)
    {
      cell = document.createElement("th");
      cell.appendChild(document.createTextNode(record[i]));
      row.appendChild(cell);
    }

    table.appendChild(thead);

    // next deal with record data if any
    tbody = document.createElement("tbody");

    for (i = 1; i < records.length; ++i)
    {
      record = records[i].split("\t");

      // skip record with missing first column
      if (record[0] == "")
        continue;

      row = document.createElement("tr");

      if (i%2)
        row.className = "odd";

      tbody.appendChild(row);

      for (j = 0; j < record.length; ++j)
      {
        cell = document.createElement("td");
        cell.appendChild(document.createTextNode(record[j]));
        row.appendChild(cell);
      }
    }

    table.appendChild(tbody);
  },

  get_site_info: function (redo)
  {
    this.query.value = "99";
    this.get_caption();

    if (!redo)
      this.push_undo();

    if (this.param.value == "")
      alert("please enter host, e.g. www.facebook.com");
    else
    {
      var req = new XMLHttpRequest();
      req.open("POST", "site.php?host="+this.param.value, true);
      req.onreadystatechange = function (evt) 
      {
        if (req.readyState == 4)
        {
          if (req.status == 200)
            dashboard.display_site_info(req);
        }
      };

      req.send(null);
    }
  },

  // data is returned as sequence of records terminated by \n
  // values separated by \t, and first record as column names
  display_site_info: function (req)
  {
    var table = document.getElementById("table");
    var data = req.responseText;
    
    var records = data.split("\n");
    var i, j, record, thead, tbody, row, cell;

    this.clear_table();

    // next deal with record data if any
    tbody = document.createElement("tbody");

    for (i = 1; i < records.length; ++i)
    {
      record = records[i].split("\t");

      // skip record with missing first column
      if (record[0] == "")
        continue;

      row = document.createElement("tr");

      if (i%2)
        row.className = "odd";

      tbody.appendChild(row);

      for (j = 0; j < record.length; ++j)
      {
        cell = document.createElement("td");
        cell.appendChild(document.createTextNode(record[j]));
        row.appendChild(cell);
      }
    }

    table.appendChild(tbody);
  },

  clear_table: function ()
  {
    var table = document.getElementById("table");
    while (table.firstChild)
      table.removeChild(table.firstChild);
  }
};

window.addEventListener("load", function() { dashboard.on_load(); }, false); 
window.addEventListener("unload", function() { dashboard.on_unload(); }, false); 
