<?php
  require("config.php");

  mb_internal_encoding("UTF-8");

  // crude check on data integrity
  // in conjunction with header
  function adler32 ($str)
  {
    $a = 1;
    $b = 0;
    $len = mb_strlen($str);

    for ($i = 0; $i < $len; $i++)
    {
      $a = ($a + ord(mb_substr($str, $i, 1))) % 65521; 
      $b = ($b + $a) % 65521;
    }

    return $a + $b * 65536;
  }

  function known_table($table)
  {
    return ($table == "relations" || $table == "parties" || $table == "site_intfo");
  }

  function validate_fields($record, $pattern)
  {
    $fields = explode(",", $record);
    $len = sizeof($fields);

    if ($len != strlen($pattern))
      return false;

    // regular expression for matching a domain name
    $reg = "/^[a-z0-9][a-z0-9\-]+[a-z0-9](\.[a-z]{2,4})+$/i";

    for ($i = 0; $i < $len; ++$i)
    {
      if ($pattern[$i] == ord('a'))
      {
        if (!preg_match($reg, trim($fields[i])))
          return false;
      }
      else if ($pattern[$i] == ord('i'))
      {
        try
        {
          $n = (int)($fields[i]);

          if ($n < 0)
            return false;
        }
        catch (Exception $e)
        {
          return false;
        }
      }
      else
        return false;
    }

    return true;
  }

  function update_relations($data)
  {
    $len = count($data) - 1;  // allowing for terminating \n

    for ($i = 0; $i < $len; $i++)
    {
      $record = $data[$i];

      if (!validate_fields($record, "aai"))
      {
        header("HTTP/1.0 400 Bad Request");
        die("400 Bad Request - malformed record");
      }

      $query = "REPLACE relations VALUES (".$record.")";

      if (!mysql_query($query))
      {
        header("HTTP/1.0 400 Bad Request");
        die("400 Bad Request - ".mysql_error());
      }
    }
  }

  function update_parties($data)
  {
    $len = count($data) - 1;  // allowing for terminating \n

    for ($i = 0; $i < $len; $i++)
    {
      $record = $data[$i];

      if (!validate_fields($record, "aai"))
      {
        header("HTTP/1.0 400 Bad Request");
        die("400 Bad Request - malformed record");
      }

      $query = "REPLACE parties VALUES (".$record.")";

      if (!mysql_query($query))
      {
        header("HTTP/1.0 400 Bad Request");
        die("400 Bad Request - ".mysql_error());
      }
    }
  }

  function max_value($a, $b)
  {
    $x = (int)$a;
    $y = (int)$b;
    return ($x > $y ? $a : $b);
  }

  // used to keep count of page loads per site
  function sum_values($a, $b)
  {
    $x = (int)$a;
    $y = (int)$b;
    return ("" + ($x + $y));
  }

  function update_site_info_record($record)
  {
    if (!validate_fields($record, "aiiiiiiiiiiiiiiiiii"))
    {
      header("HTTP/1.0 400 Bad Request");
      die("400 Bad Request - malformed record");
    }

    // extract host name from first field
    $fields = explode(",", $record);
    $host = $fields[0];

    // use it to retrieve existing record if one exists
    $query = "SELECT ALL FROM site_info WHERE host='".mysql_real_escape_string($host)."'";

    $result = mysql_query($query);
    $row = mysql_fetch_array($result);

    if ($row)
    {
      $visited = sum_values($fields[1], $row["visited"]);
      $session_cookies = max_value($fields[2], $row["session_cookies"]);
      $lasting_cookies = max_value($fields[3], $row["lasting_cookies"]);
      $flash_cookies = max_value($fields[4], $row["lasting_cookies"]);
      $int_3rd_parties = max_value($fields[5], $row["int_3rd_parties"]);
      $ext_3rd_parties = max_value($fields[6], $row["ext_3rd_parties"]);
      $int_3rd_party_session_cookies =
         max_value($fields[7], $row["int_3rd_party_session_cookies"]);
      $int_3rd_party_lasting_cookies =
         max_value($fields[8], $row["int_3rd_party_lasting_cookies"]);
      $int_3rd_party_flash_cookies =
         max_value($fields[9], $row["int_3rd_party_flash_cookies"]);
      $ext_3rd_party_session_cookies =
         max_value($fields[10], $row["ext_3rd_party_session_cookies"]);
      $ext_3rd_party_lasting_cookies =
         max_value($fields[11], $row["ext_3rd_party_lasting_cookies"]);
      $ext_3rd_party_flash_cookies =
         max_value($fields[12], $row["ext_3rd_party_flash_cookies"]);
      $dom_storage = max_value($fields[13], $row["dom_storage"]);
      $html5_pings = max_value($fields[14], $row["html5_pings"]);
      $invisible_images = max_value($fields[15], $row["invisible_images"]);
      $suspicious_urls = max_value($fields[16], $row["suspicious_urls"]);
      $geo_permission = max_value($fields[17], $row["geo_permission"]);
      $p3p = max_value($fields[18], $row["p3p"]);

      $record = implode(",", Array($host, $session_cookies, $lasting_cookies,
        $int_3rd_parties,$ext_3rd_parties, $int_3rd_party_session_cookies,
        $int_3rd_party_lasting_cookies, $int_3rd_party_flash_cookies,
        $ext_3rd_party_session_cookies, $ext_3rd_party_lasting_cookies,
        $ext_3rd_party_flash_cookies, $dom_storage, $html5_pings,
        $invisible_images, $suspicious_urls, $geo_permission, $p3p));

      $query = "REPLACE site_info VALUES (".$record.")";

      if (!mysql_query($query))
      {
        header("HTTP/1.0 400 Bad Request");
        die("400 Bad Request - ".mysql_error());
      }
    }
    else
    {
      $query = "INSERT site_info VALUES (".$record.")";

      if (!mysql_query($query))
      {
        header("HTTP/1.0 400 Bad Request");
        die("400 Bad Request - ".mysql_error());
      }
    }
  }

  function update_site_info($data)
  {
    $len = count($data) - 1;  // allowing for terminating \n
    $expected = 19;

    for ($i = 0; $i < $len; $i++)
    {
      $record = $data[$i];
      $fields = substr_count($record, ",") + 1;

      if ($expected != $fields)
      {
        header("HTTP/1.0 400 Bad Request");
        die("400 Bad Request - malformed record");
      }

      update_site_info_record($record);
    }
  }

  $nl = "\n";
  $headers = apache_request_headers();
  $body = file_get_contents('php://input');
  $checksum = $headers["X-Data-Check"];
  $table = $headers["Dashboard-Table"];

  $type = $headers["Content-Type"];
  $mime = explode(';', $type);

  if ($mime[0] != "text/plain" ||
      !$checksum ||
      adler32($body) != $checksum ||
      !known_table($table))
  {
    header("HTTP/1.0 400 Bad Request");
    die("Bad Request");
  }

  $data = explode("\n", $body);

  // $database, $username and $password set in config.php
  $db = mysql_connect($db_master, $username, $password);

  if (!$db)
  {
    header("HTTP/1.0 500 Internal Error");
    die("500 Internal Error - Couldn't open connection to MYSQL: ".mysql_error());
  }

  if (!mysql_select_db($database))
  {
    header("HTTP/1.0 500 Internal Error");
    die( "500 Internal Error - Unable to select database");
  }

  if ($table == "relations")
    update_relations($data);
  if ($table == "parties")
    update_parties($data);
  else if ($table == "site_info")
    update_site_info($data);
  
  mysql_close($db);
  
  header('Content-Type: text/html');
?>
<html>
<title>Thanks for sharing</title>
<h1>Thanks for sharing</h1>
<p>Thanks for helping to make the Web a little bit more transparent.</p>
</html>

